import { TypeRegistry, FormatRegistry } from '../type/registry/index.mjs';
import { Unsafe } from '../type/unsafe/index.mjs';
import { Kind } from '../type/symbols/index.mjs';
export class TypeSystemDuplicateTypeKind extends Error {
    constructor(kind) {
        super(`Duplicate type kind '${kind}' detected`);
    }
}
export class TypeSystemDuplicateFormat extends Error {
    constructor(kind) {
        super(`Duplicate string format '${kind}' detected`);
    }
}
export var TypeSystem;
(function (TypeSystem) {
    function Type(kind, check) {
        if (TypeRegistry.Has(kind))
            throw new TypeSystemDuplicateTypeKind(kind);
        TypeRegistry.Set(kind, check);
        return (options = {}) => Unsafe({ ...options, [Kind]: kind });
    }
    TypeSystem.Type = Type;
    function Format(format, check) {
        if (FormatRegistry.Has(format))
            throw new TypeSystemDuplicateFormat(format);
        FormatRegistry.Set(format, check);
        return format;
    }
    TypeSystem.Format = Format;
})(TypeSystem || (TypeSystem = {}));
