import { CloneType } from '../clone/type.mjs';
import { Discard } from '../discard/index.mjs';
import { IsUndefined } from '../guard/value.mjs';
import { TConstructor as IsConstructorType, TFunction as IsFunctionType, TIntersect as IsIntersectType, TUnion as IsUnionType, TTuple as IsTupleType, TArray as IsArrayType, TObject as IsObjectType, TPromise as IsPromiseType, TAsyncIterator as IsAsyncIteratorType, TIterator as IsIteratorType, TRef as IsRefType, } from '../guard/type.mjs';
function FromRest(schema, references) {
    return schema.map((schema) => Deref(schema, references));
}
function FromProperties(properties, references) {
    return globalThis.Object.getOwnPropertyNames(properties).reduce((acc, key) => {
        return { ...acc, [key]: Deref(properties[key], references) };
    }, {});
}
function FromConstructor(schema, references) {
    const clone = CloneType(schema);
    clone.parameters = FromRest(clone.parameters, references);
    clone.returns = Deref(clone.returns, references);
    return clone;
}
function FromFunction(schema, references) {
    const clone = CloneType(schema);
    clone.parameters = FromRest(clone.parameters, references);
    clone.returns = Deref(clone.returns, references);
    return clone;
}
function FromIntersect(schema, references) {
    const clone = CloneType(schema);
    clone.allOf = FromRest(clone.allOf, references);
    return clone;
}
function FromUnion(schema, references) {
    const clone = CloneType(schema);
    clone.anyOf = FromRest(clone.anyOf, references);
    return clone;
}
function FromTuple(schema, references) {
    const clone = CloneType(schema);
    if (IsUndefined(clone.items))
        return clone;
    clone.items = FromRest(clone.items, references);
    return clone;
}
function FromArray(schema, references) {
    const clone = CloneType(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
function FromObject(schema, references) {
    const clone = CloneType(schema);
    clone.properties = FromProperties(clone.properties, references);
    return clone;
}
function FromPromise(schema, references) {
    const clone = CloneType(schema);
    clone.item = Deref(clone.item, references);
    return clone;
}
function FromAsyncIterator(schema, references) {
    const clone = CloneType(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
function FromIterator(schema, references) {
    const clone = CloneType(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
function FromRef(schema, references) {
    const target = references.find(remote => remote.$id === schema.$ref);
    if (target === undefined)
        throw Error(`Unable to dereference schema with $id ${schema.$ref}`);
    const discard = Discard(target, ['$id']);
    return Deref(discard, references);
}
export function DerefResolve(schema, references) {
    return (IsConstructorType(schema) ? FromConstructor(schema, references) :
        IsFunctionType(schema) ? FromFunction(schema, references) :
            IsIntersectType(schema) ? FromIntersect(schema, references) :
                IsUnionType(schema) ? FromUnion(schema, references) :
                    IsTupleType(schema) ? FromTuple(schema, references) :
                        IsArrayType(schema) ? FromArray(schema, references) :
                            IsObjectType(schema) ? FromObject(schema, references) :
                                IsPromiseType(schema) ? FromPromise(schema, references) :
                                    IsAsyncIteratorType(schema) ? FromAsyncIterator(schema, references) :
                                        IsIteratorType(schema) ? FromIterator(schema, references) :
                                            IsRefType(schema) ? FromRef(schema, references) :
                                                schema);
}
export function Deref(schema, references) {
    return DerefResolve(schema, references);
}
