import { Any } from '../any/index.mjs';
import { Function } from '../function/index.mjs';
import { Number } from '../number/index.mjs';
import { String } from '../string/index.mjs';
import { Unknown } from '../unknown/index.mjs';
import { TemplateLiteralToUnion } from '../template-literal/index.mjs';
import { PatternNumberExact, PatternStringExact } from '../patterns/index.mjs';
import { Kind, Hint } from '../symbols/index.mjs';
import { TypeGuard, ValueGuard } from '../guard/index.mjs';
export class ExtendsResolverError extends Error {
}
export var ExtendsResult;
(function (ExtendsResult) {
    ExtendsResult[ExtendsResult["Union"] = 0] = "Union";
    ExtendsResult[ExtendsResult["True"] = 1] = "True";
    ExtendsResult[ExtendsResult["False"] = 2] = "False";
})(ExtendsResult || (ExtendsResult = {}));
function IntoBooleanResult(result) {
    return result === ExtendsResult.False ? result : ExtendsResult.True;
}
function Throw(message) {
    throw new ExtendsResolverError(message);
}
function IsStructuralRight(right) {
    return (TypeGuard.TNever(right) ||
        TypeGuard.TIntersect(right) ||
        TypeGuard.TUnion(right) ||
        TypeGuard.TUnknown(right) ||
        TypeGuard.TAny(right));
}
function StructuralRight(left, right) {
    return (TypeGuard.TNever(right) ? TNeverRight(left, right) :
        TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
            TypeGuard.TUnion(right) ? TUnionRight(left, right) :
                TypeGuard.TUnknown(right) ? TUnknownRight(left, right) :
                    TypeGuard.TAny(right) ? TAnyRight(left, right) :
                        Throw('StructuralRight'));
}
function TAnyRight(left, right) {
    return ExtendsResult.True;
}
function TAny(left, right) {
    return (TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
        (TypeGuard.TUnion(right) && right.anyOf.some((schema) => TypeGuard.TAny(schema) || TypeGuard.TUnknown(schema))) ? ExtendsResult.True :
            TypeGuard.TUnion(right) ? ExtendsResult.Union :
                TypeGuard.TUnknown(right) ? ExtendsResult.True :
                    TypeGuard.TAny(right) ? ExtendsResult.True :
                        ExtendsResult.Union);
}
function TArrayRight(left, right) {
    return (TypeGuard.TUnknown(left) ? ExtendsResult.False :
        TypeGuard.TAny(left) ? ExtendsResult.Union :
            TypeGuard.TNever(left) ? ExtendsResult.True :
                ExtendsResult.False);
}
function TArray(left, right) {
    return (TypeGuard.TObject(right) && IsObjectArrayLike(right) ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            !TypeGuard.TArray(right) ? ExtendsResult.False :
                IntoBooleanResult(Visit(left.items, right.items)));
}
function TAsyncIterator(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        !TypeGuard.TAsyncIterator(right) ? ExtendsResult.False :
            IntoBooleanResult(Visit(left.items, right.items)));
}
function TBigInt(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TBigInt(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TBooleanRight(left, right) {
    return (TypeGuard.TLiteralBoolean(left) ? ExtendsResult.True :
        TypeGuard.TBoolean(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TBoolean(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TBoolean(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TConstructor(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            !TypeGuard.TConstructor(right) ? ExtendsResult.False :
                left.parameters.length > right.parameters.length ? ExtendsResult.False :
                    (!left.parameters.every((schema, index) => IntoBooleanResult(Visit(right.parameters[index], schema)) === ExtendsResult.True)) ? ExtendsResult.False :
                        IntoBooleanResult(Visit(left.returns, right.returns)));
}
function TDate(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TDate(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TFunction(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            !TypeGuard.TFunction(right) ? ExtendsResult.False :
                left.parameters.length > right.parameters.length ? ExtendsResult.False :
                    (!left.parameters.every((schema, index) => IntoBooleanResult(Visit(right.parameters[index], schema)) === ExtendsResult.True)) ? ExtendsResult.False :
                        IntoBooleanResult(Visit(left.returns, right.returns)));
}
function TIntegerRight(left, right) {
    return (TypeGuard.TLiteral(left) && ValueGuard.IsNumber(left.const) ? ExtendsResult.True :
        TypeGuard.TNumber(left) || TypeGuard.TInteger(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TInteger(left, right) {
    return (TypeGuard.TInteger(right) || TypeGuard.TNumber(right) ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            TypeGuard.TObject(right) ? TObjectRight(left, right) :
                TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                    ExtendsResult.False);
}
function TIntersectRight(left, right) {
    return right.allOf.every((schema) => Visit(left, schema) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TIntersect(left, right) {
    return left.allOf.some((schema) => Visit(schema, right) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TIterator(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        !TypeGuard.TIterator(right) ? ExtendsResult.False :
            IntoBooleanResult(Visit(left.items, right.items)));
}
function TLiteral(left, right) {
    return (TypeGuard.TLiteral(right) && right.const === left.const ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            TypeGuard.TObject(right) ? TObjectRight(left, right) :
                TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                    TypeGuard.TString(right) ? TStringRight(left, right) :
                        TypeGuard.TNumber(right) ? TNumberRight(left, right) :
                            TypeGuard.TInteger(right) ? TIntegerRight(left, right) :
                                TypeGuard.TBoolean(right) ? TBooleanRight(left, right) :
                                    ExtendsResult.False);
}
function TNeverRight(left, right) {
    return ExtendsResult.False;
}
function TNever(left, right) {
    return ExtendsResult.True;
}
function UnwrapTNot(schema) {
    let [current, depth] = [schema, 0];
    while (true) {
        if (!TypeGuard.TNot(current))
            break;
        current = current.not;
        depth += 1;
    }
    return depth % 2 === 0 ? current : Unknown();
}
function TNot(left, right) {
    return (TypeGuard.TNot(left) ? Visit(UnwrapTNot(left), right) :
        TypeGuard.TNot(right) ? Visit(left, UnwrapTNot(right)) :
            Throw('Invalid fallthrough for Not'));
}
function TNull(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TNull(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TNumberRight(left, right) {
    return (TypeGuard.TLiteralNumber(left) ? ExtendsResult.True :
        TypeGuard.TNumber(left) || TypeGuard.TInteger(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TNumber(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TInteger(right) || TypeGuard.TNumber(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function IsObjectPropertyCount(schema, count) {
    return Object.getOwnPropertyNames(schema.properties).length === count;
}
function IsObjectStringLike(schema) {
    return IsObjectArrayLike(schema);
}
function IsObjectSymbolLike(schema) {
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'description' in schema.properties && TypeGuard.TUnion(schema.properties.description) && schema.properties.description.anyOf.length === 2 && ((TypeGuard.TString(schema.properties.description.anyOf[0]) &&
        TypeGuard.TUndefined(schema.properties.description.anyOf[1])) || (TypeGuard.TString(schema.properties.description.anyOf[1]) &&
        TypeGuard.TUndefined(schema.properties.description.anyOf[0]))));
}
function IsObjectNumberLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectBooleanLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectBigIntLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectDateLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectUint8ArrayLike(schema) {
    return IsObjectArrayLike(schema);
}
function IsObjectFunctionLike(schema) {
    const length = Number();
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'length' in schema.properties && IntoBooleanResult(Visit(schema.properties['length'], length)) === ExtendsResult.True);
}
function IsObjectConstructorLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectArrayLike(schema) {
    const length = Number();
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'length' in schema.properties && IntoBooleanResult(Visit(schema.properties['length'], length)) === ExtendsResult.True);
}
function IsObjectPromiseLike(schema) {
    const then = Function([Any()], Any());
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'then' in schema.properties && IntoBooleanResult(Visit(schema.properties['then'], then)) === ExtendsResult.True);
}
function Property(left, right) {
    return (Visit(left, right) === ExtendsResult.False ? ExtendsResult.False :
        TypeGuard.TOptional(left) && !TypeGuard.TOptional(right) ? ExtendsResult.False :
            ExtendsResult.True);
}
function TObjectRight(left, right) {
    return (TypeGuard.TUnknown(left) ? ExtendsResult.False :
        TypeGuard.TAny(left) ? ExtendsResult.Union : (TypeGuard.TNever(left) ||
            (TypeGuard.TLiteralString(left) && IsObjectStringLike(right)) ||
            (TypeGuard.TLiteralNumber(left) && IsObjectNumberLike(right)) ||
            (TypeGuard.TLiteralBoolean(left) && IsObjectBooleanLike(right)) ||
            (TypeGuard.TSymbol(left) && IsObjectSymbolLike(right)) ||
            (TypeGuard.TBigInt(left) && IsObjectBigIntLike(right)) ||
            (TypeGuard.TString(left) && IsObjectStringLike(right)) ||
            (TypeGuard.TSymbol(left) && IsObjectSymbolLike(right)) ||
            (TypeGuard.TNumber(left) && IsObjectNumberLike(right)) ||
            (TypeGuard.TInteger(left) && IsObjectNumberLike(right)) ||
            (TypeGuard.TBoolean(left) && IsObjectBooleanLike(right)) ||
            (TypeGuard.TUint8Array(left) && IsObjectUint8ArrayLike(right)) ||
            (TypeGuard.TDate(left) && IsObjectDateLike(right)) ||
            (TypeGuard.TConstructor(left) && IsObjectConstructorLike(right)) ||
            (TypeGuard.TFunction(left) && IsObjectFunctionLike(right))) ? ExtendsResult.True :
            (TypeGuard.TRecord(left) && TypeGuard.TString(RecordKey(left))) ? (() => {
                return right[Hint] === 'Record' ? ExtendsResult.True : ExtendsResult.False;
            })() :
                (TypeGuard.TRecord(left) && TypeGuard.TNumber(RecordKey(left))) ? (() => {
                    return IsObjectPropertyCount(right, 0) ? ExtendsResult.True : ExtendsResult.False;
                })() :
                    ExtendsResult.False);
}
function TObject(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TRecord(right) ? TRecordRight(left, right) :
            !TypeGuard.TObject(right) ? ExtendsResult.False :
                (() => {
                    for (const key of Object.getOwnPropertyNames(right.properties)) {
                        if (!(key in left.properties) && !TypeGuard.TOptional(right.properties[key])) {
                            return ExtendsResult.False;
                        }
                        if (TypeGuard.TOptional(right.properties[key])) {
                            return ExtendsResult.True;
                        }
                        if (Property(left.properties[key], right.properties[key]) === ExtendsResult.False) {
                            return ExtendsResult.False;
                        }
                    }
                    return ExtendsResult.True;
                })());
}
function TPromise(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) && IsObjectPromiseLike(right) ? ExtendsResult.True :
            !TypeGuard.TPromise(right) ? ExtendsResult.False :
                IntoBooleanResult(Visit(left.item, right.item)));
}
function RecordKey(schema) {
    return (PatternNumberExact in schema.patternProperties ? Number() :
        PatternStringExact in schema.patternProperties ? String() :
            Throw('Unknown record key pattern'));
}
function RecordValue(schema) {
    return (PatternNumberExact in schema.patternProperties ? schema.patternProperties[PatternNumberExact] :
        PatternStringExact in schema.patternProperties ? schema.patternProperties[PatternStringExact] :
            Throw('Unable to get record value schema'));
}
function TRecordRight(left, right) {
    const [Key, Value] = [RecordKey(right), RecordValue(right)];
    return ((TypeGuard.TLiteralString(left) && TypeGuard.TNumber(Key) && IntoBooleanResult(Visit(left, Value)) === ExtendsResult.True) ? ExtendsResult.True :
        TypeGuard.TUint8Array(left) && TypeGuard.TNumber(Key) ? Visit(left, Value) :
            TypeGuard.TString(left) && TypeGuard.TNumber(Key) ? Visit(left, Value) :
                TypeGuard.TArray(left) && TypeGuard.TNumber(Key) ? Visit(left, Value) :
                    TypeGuard.TObject(left) ? (() => {
                        for (const key of Object.getOwnPropertyNames(left.properties)) {
                            if (Property(Value, left.properties[key]) === ExtendsResult.False) {
                                return ExtendsResult.False;
                            }
                        }
                        return ExtendsResult.True;
                    })() :
                        ExtendsResult.False);
}
function TRecord(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            !TypeGuard.TRecord(right) ? ExtendsResult.False :
                Visit(RecordValue(left), RecordValue(right)));
}
function TStringRight(left, right) {
    return (TypeGuard.TLiteral(left) && ValueGuard.IsString(left.const) ? ExtendsResult.True :
        TypeGuard.TString(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TString(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TString(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TSymbol(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TSymbol(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TTemplateLiteral(left, right) {
    return (TypeGuard.TTemplateLiteral(left) ? Visit(TemplateLiteralToUnion(left), right) :
        TypeGuard.TTemplateLiteral(right) ? Visit(left, TemplateLiteralToUnion(right)) :
            Throw('Invalid fallthrough for TemplateLiteral'));
}
function IsArrayOfTuple(left, right) {
    return (TypeGuard.TArray(right) &&
        left.items !== undefined &&
        left.items.every((schema) => Visit(schema, right.items) === ExtendsResult.True));
}
function TTupleRight(left, right) {
    return (TypeGuard.TNever(left) ? ExtendsResult.True :
        TypeGuard.TUnknown(left) ? ExtendsResult.False :
            TypeGuard.TAny(left) ? ExtendsResult.Union :
                ExtendsResult.False);
}
function TTuple(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) && IsObjectArrayLike(right) ? ExtendsResult.True :
            TypeGuard.TArray(right) && IsArrayOfTuple(left, right) ? ExtendsResult.True :
                !TypeGuard.TTuple(right) ? ExtendsResult.False :
                    (ValueGuard.IsUndefined(left.items) && !ValueGuard.IsUndefined(right.items)) || (!ValueGuard.IsUndefined(left.items) && ValueGuard.IsUndefined(right.items)) ? ExtendsResult.False :
                        (ValueGuard.IsUndefined(left.items) && !ValueGuard.IsUndefined(right.items)) ? ExtendsResult.True :
                            left.items.every((schema, index) => Visit(schema, right.items[index]) === ExtendsResult.True) ? ExtendsResult.True :
                                ExtendsResult.False);
}
function TUint8Array(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TUint8Array(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TUndefined(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.TObject(right) ? TObjectRight(left, right) :
            TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                TypeGuard.TVoid(right) ? VoidRight(left, right) :
                    TypeGuard.TUndefined(right) ? ExtendsResult.True :
                        ExtendsResult.False);
}
function TUnionRight(left, right) {
    return right.anyOf.some((schema) => Visit(left, schema) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TUnion(left, right) {
    return left.anyOf.every((schema) => Visit(schema, right) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TUnknownRight(left, right) {
    return ExtendsResult.True;
}
function TUnknown(left, right) {
    return (TypeGuard.TNever(right) ? TNeverRight(left, right) :
        TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
            TypeGuard.TUnion(right) ? TUnionRight(left, right) :
                TypeGuard.TAny(right) ? TAnyRight(left, right) :
                    TypeGuard.TString(right) ? TStringRight(left, right) :
                        TypeGuard.TNumber(right) ? TNumberRight(left, right) :
                            TypeGuard.TInteger(right) ? TIntegerRight(left, right) :
                                TypeGuard.TBoolean(right) ? TBooleanRight(left, right) :
                                    TypeGuard.TArray(right) ? TArrayRight(left, right) :
                                        TypeGuard.TTuple(right) ? TTupleRight(left, right) :
                                            TypeGuard.TObject(right) ? TObjectRight(left, right) :
                                                TypeGuard.TUnknown(right) ? ExtendsResult.True :
                                                    ExtendsResult.False);
}
function VoidRight(left, right) {
    return (TypeGuard.TUndefined(left) ? ExtendsResult.True :
        TypeGuard.TUndefined(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TVoid(left, right) {
    return (TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
        TypeGuard.TUnion(right) ? TUnionRight(left, right) :
            TypeGuard.TUnknown(right) ? TUnknownRight(left, right) :
                TypeGuard.TAny(right) ? TAnyRight(left, right) :
                    TypeGuard.TObject(right) ? TObjectRight(left, right) :
                        TypeGuard.TVoid(right) ? ExtendsResult.True :
                            ExtendsResult.False);
}
function Visit(left, right) {
    return ((TypeGuard.TTemplateLiteral(left) || TypeGuard.TTemplateLiteral(right)) ? TTemplateLiteral(left, right) :
        (TypeGuard.TNot(left) || TypeGuard.TNot(right)) ? TNot(left, right) :
            TypeGuard.TAny(left) ? TAny(left, right) :
                TypeGuard.TArray(left) ? TArray(left, right) :
                    TypeGuard.TBigInt(left) ? TBigInt(left, right) :
                        TypeGuard.TBoolean(left) ? TBoolean(left, right) :
                            TypeGuard.TAsyncIterator(left) ? TAsyncIterator(left, right) :
                                TypeGuard.TConstructor(left) ? TConstructor(left, right) :
                                    TypeGuard.TDate(left) ? TDate(left, right) :
                                        TypeGuard.TFunction(left) ? TFunction(left, right) :
                                            TypeGuard.TInteger(left) ? TInteger(left, right) :
                                                TypeGuard.TIntersect(left) ? TIntersect(left, right) :
                                                    TypeGuard.TIterator(left) ? TIterator(left, right) :
                                                        TypeGuard.TLiteral(left) ? TLiteral(left, right) :
                                                            TypeGuard.TNever(left) ? TNever(left, right) :
                                                                TypeGuard.TNull(left) ? TNull(left, right) :
                                                                    TypeGuard.TNumber(left) ? TNumber(left, right) :
                                                                        TypeGuard.TObject(left) ? TObject(left, right) :
                                                                            TypeGuard.TRecord(left) ? TRecord(left, right) :
                                                                                TypeGuard.TString(left) ? TString(left, right) :
                                                                                    TypeGuard.TSymbol(left) ? TSymbol(left, right) :
                                                                                        TypeGuard.TTuple(left) ? TTuple(left, right) :
                                                                                            TypeGuard.TPromise(left) ? TPromise(left, right) :
                                                                                                TypeGuard.TUint8Array(left) ? TUint8Array(left, right) :
                                                                                                    TypeGuard.TUndefined(left) ? TUndefined(left, right) :
                                                                                                        TypeGuard.TUnion(left) ? TUnion(left, right) :
                                                                                                            TypeGuard.TUnknown(left) ? TUnknown(left, right) :
                                                                                                                TypeGuard.TVoid(left) ? TVoid(left, right) :
                                                                                                                    Throw(`Unknown left type operand '${left[Kind]}'`));
}
export function ExtendsCheck(left, right) {
    return Visit(left, right);
}
