import { Never } from '../never/index.mjs';
import { IntersectResolve } from '../intersect/index.mjs';
import { UnionResolve } from '../union/index.mjs';
import { TIntersect as IsIntersectType, TUnion as IsUnionType, TTuple as IsTupleType, TArray as IsArrayType, TObject as IsObjectType, TNever as IsNeverType } from '../guard/type.mjs';
function FromRest(T, K) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [FromKey(L, K), ...FromRest(R, K)]
        : []);
}
function FromIntersectRest(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsNeverType(L)
            ? [...FromIntersectRest(R)]
            : [L, ...FromIntersectRest(R)]
        : []);
}
function FromIntersect(T, K) {
    return (IntersectResolve(FromIntersectRest(FromRest(T, K))));
}
function FromUnionRest(T, S = T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsNeverType(L)
            ? []
            : FromUnionRest(R, S)
        : S);
}
function FromUnion(T, K) {
    return (UnionResolve(FromUnionRest(FromRest(T, K))));
}
function FromTuple(T, K) {
    return (K in T ? T[K] :
        K === '[number]' ? UnionResolve(T) :
            Never());
}
function FromArray(T, K) {
    return (K === '[number]'
        ? T
        : Never());
}
function FromProperty(T, K) {
    return (K in T
        ? T[K]
        : Never());
}
function FromKey(T, K) {
    return (IsIntersectType(T) ? FromIntersect(T.allOf, K) :
        IsUnionType(T) ? FromUnion(T.anyOf, K) :
            IsTupleType(T) ? FromTuple(T.items ?? [], K) :
                IsArrayType(T) ? FromArray(T.items, K) :
                    IsObjectType(T) ? FromProperty(T.properties, K) :
                        Never());
}
function FromKeys(T, K) {
    const [L, ...R] = K;
    return (K.length > 0
        ? [FromKey(T, L), ...FromKeys(T, R)]
        : []);
}
export function IndexedTypeResolve(T, K) {
    return (UnionResolve(FromKeys(T, K)));
}
