import { Never } from '../never/index.mjs';
import { OptionalFromIntersect } from '../modifiers/index.mjs';
import { Kind } from '../symbols/index.mjs';
import { CloneType, CloneRest } from '../clone/type.mjs';
import { TTransform as IsTransformType, TObject as IsObjectType, TSchema as IsSchemaType } from '../guard/type.mjs';
export function IntersectResolve(T) {
    return (T.length === 0
        ? Never()
        : T.length === 1
            ? T[0]
            : OptionalFromIntersect(T));
}
export function Intersect(T, options = {}) {
    if (T.length === 0)
        return Never();
    if (T.length === 1)
        return CloneType(T[0], options);
    if (T.some((schema) => IsTransformType(schema)))
        throw new Error('Cannot intersect transform types');
    const allObjects = T.every((schema) => IsObjectType(schema));
    const clonedUnevaluatedProperties = IsSchemaType(options.unevaluatedProperties) ? { unevaluatedProperties: CloneType(options.unevaluatedProperties) } : {};
    return (options.unevaluatedProperties === false || IsSchemaType(options.unevaluatedProperties) || allObjects
        ? { ...options, ...clonedUnevaluatedProperties, [Kind]: 'Intersect', type: 'object', allOf: CloneRest(T) }
        : { ...options, ...clonedUnevaluatedProperties, [Kind]: 'Intersect', allOf: CloneRest(T) });
}
