import { TemplateLiteral, TemplateLiteralParseExact, IsTemplateLiteralFinite, TemplateLiteralGenerate } from '../template-literal/index.mjs';
import { Literal } from '../literal/index.mjs';
import { Union } from '../union/index.mjs';
import { TTemplateLiteral as IsTemplateLiteralType, TUnion as IsUnionType, TLiteral as IsLiteralType } from '../guard/type.mjs';
import { CloneType } from '../clone/type.mjs';
function ApplyUncapitalize(value) {
    const [first, rest] = [value.slice(0, 1), value.slice(1)];
    return [first.toLowerCase(), rest].join('');
}
function ApplyCapitalize(value) {
    const [first, rest] = [value.slice(0, 1), value.slice(1)];
    return [first.toUpperCase(), rest].join('');
}
function ApplyUppercase(value) {
    return value.toUpperCase();
}
function ApplyLowercase(value) {
    return value.toLowerCase();
}
function FromTemplateLiteral(schema, mode) {
    const expression = TemplateLiteralParseExact(schema.pattern);
    const finite = IsTemplateLiteralFinite(expression);
    if (!finite)
        return { ...schema, pattern: FromLiteralValue(schema.pattern, mode) };
    const strings = [...TemplateLiteralGenerate(expression)];
    const literals = strings.map((value) => Literal(value));
    const mapped = FromRest(literals, mode);
    const union = Union(mapped);
    return TemplateLiteral([union]);
}
function FromLiteralValue(value, mode) {
    return (typeof value === 'string' ? (mode === 'Uncapitalize' ? ApplyUncapitalize(value) :
        mode === 'Capitalize' ? ApplyCapitalize(value) :
            mode === 'Uppercase' ? ApplyUppercase(value) :
                mode === 'Lowercase' ? ApplyLowercase(value) :
                    value) : value.toString());
}
function FromRest(T, mode) {
    const [L, ...R] = T;
    return (T.length > 0 ? [IntrinsicResolve(L, mode), ...FromRest(R, mode)] : []);
}
export function IntrinsicResolve(schema, mode) {
    return (IsTemplateLiteralType(schema) ? FromTemplateLiteral(schema, mode) :
        IsUnionType(schema) ? Union(FromRest(schema.anyOf, mode)) :
            IsLiteralType(schema) ? Literal(FromLiteralValue(schema.const, mode)) :
                schema);
}
export function Uncapitalize(T, options = {}) {
    return CloneType(IntrinsicResolve(T, 'Uncapitalize'), options);
}
export function Uppercase(T, options = {}) {
    return CloneType(IntrinsicResolve(T, 'Uppercase'), options);
}
export function Lowercase(T, options = {}) {
    return CloneType(IntrinsicResolve(T, 'Lowercase'), options);
}
export function Capitalize(T, options = {}) {
    return CloneType(IntrinsicResolve(T, 'Capitalize'), options);
}
