import { Increment } from '../helpers/index.mjs';
import { OperatorUnionMany, OperatorIntersectMany } from '../operators/index.mjs';
import { TIntersect as IsIntersectType, TUnion as IsUnionType, TTuple as IsTupleType, TArray as IsArrayType, TObject as IsObjectType, TRecord as IsRecordType } from '../guard/type.mjs';
function FromRest(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [KeyOfStringResolve(L), ...FromRest(R)]
        : []);
}
function FromIntersect(T, C = FromRest(T)) {
    return OperatorUnionMany(C);
}
function FromUnion(T) {
    const C = FromRest(T);
    return OperatorIntersectMany(C);
}
function FromTuple(T, I = '0') {
    const [_, ...R] = T;
    return (T.length > 0
        ? [I, ...FromTuple(R, Increment(I))]
        : []);
}
function FromArray(_) {
    return (['[number]']);
}
function FromProperties(T) {
    return (globalThis.Object.getOwnPropertyNames(T));
}
function FromPatternProperties(patternProperties) {
    if (!includePatternProperties)
        return [];
    const patternPropertyKeys = globalThis.Object.getOwnPropertyNames(patternProperties);
    return patternPropertyKeys.map(key => {
        return (key[0] === '^' && key[key.length - 1] === '$')
            ? key.slice(1, key.length - 1)
            : key;
    });
}
export function KeyOfStringResolve(T) {
    return (IsIntersectType(T) ? FromIntersect(T.allOf) :
        IsUnionType(T) ? FromUnion(T.anyOf) :
            IsTupleType(T) ? FromTuple(T.items ?? []) :
                IsArrayType(T) ? FromArray(T.items) :
                    IsObjectType(T) ? FromProperties(T.properties) :
                        IsRecordType(T) ? FromPatternProperties(T.patternProperties) :
                            []);
}
let includePatternProperties = false;
export function KeyOfStringResolvePattern(schema) {
    includePatternProperties = true;
    const keys = KeyOfStringResolve(schema);
    includePatternProperties = false;
    const pattern = keys.map((key) => `(${key})`);
    return `^(${pattern.join('|')})$`;
}
