import { Intersect } from '../intersect/index.mjs';
import { Union } from '../union/index.mjs';
import { Object } from '../object/index.mjs';
import { IndexedKeyResolve } from '../indexed/index.mjs';
import { Discard } from '../discard/index.mjs';
import { TransformKind } from '../symbols/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { TIntersect as IsIntersectType, TUnion as IsUnionType, TObject as IsObjectType, TSchema as IsSchemaType } from '../guard/type.mjs';
function FromIntersect(T, K) {
    return T.map((T) => OmitResolve(T, K));
}
function FromUnion(T, K) {
    return T.map((T) => OmitResolve(T, K));
}
function FromProperty(T, K) {
    const { [K]: _, ...R } = T;
    return R;
}
function FromProperties(T, K) {
    return K.reduce((T, K2) => {
        return FromProperty(T, K2);
    }, T);
}
export function OmitResolve(T, K) {
    return (IsIntersectType(T) ? Intersect(FromIntersect(T.allOf, K)) :
        IsUnionType(T) ? Union(FromUnion(T.anyOf, K)) :
            IsObjectType(T) ? Object(FromProperties(T.properties, K)) :
                Object({}));
}
export function Omit(T, K, options = {}) {
    const I = IsSchemaType(K) ? IndexedKeyResolve(K) : K;
    const D = Discard(T, [TransformKind, '$id', 'required']);
    const R = CloneType(OmitResolve(T, I), options);
    return { ...D, ...R };
}
