import { Optional } from '../optional/index.mjs';
import { Object } from '../object/index.mjs';
import { Intersect } from '../intersect/index.mjs';
import { Union } from '../union/index.mjs';
import { Discard } from '../discard/index.mjs';
import { TransformKind } from '../symbols/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { TIntersect as IsIntersectType, TUnion as IsUnionType, TObject as IsObjectType } from '../guard/type.mjs';
function FromIntersect(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [PartialResolve(L), ...FromIntersect(R)]
        : []);
}
function FromUnion(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [PartialResolve(L), ...FromUnion(R)]
        : []);
}
function FromProperties(T) {
    return globalThis.Object.getOwnPropertyNames(T).reduce((Acc, K) => {
        return { ...Acc, [K]: Optional(T[K]) };
    }, {});
}
export function PartialResolve(T) {
    return (IsIntersectType(T) ? Intersect(FromIntersect(T.allOf)) :
        IsUnionType(T) ? Union(FromUnion(T.anyOf)) :
            IsObjectType(T) ? Object(FromProperties(T.properties)) :
                Object({}));
}
export function Partial(T, options = {}) {
    const D = Discard(T, [TransformKind, '$id', 'required']);
    const R = CloneType(PartialResolve(T), options);
    return { ...D, ...R };
}
