import { Intersect } from '../intersect/index.mjs';
import { Union } from '../union/index.mjs';
import { Object } from '../object/index.mjs';
import { IndexedKeyResolve } from '../indexed/index.mjs';
import { Discard } from '../discard/index.mjs';
import { TransformKind } from '../symbols/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { TIntersect as IsIntersectType, TUnion as IsUnionType, TObject as IsObjectType, TSchema as IsSchemaType } from '../guard/type.mjs';
function FromIntersect(T, K) {
    return T.map((T) => PickResolve(T, K));
}
function FromUnion(T, K) {
    return T.map((T) => PickResolve(T, K));
}
function FromProperties(T, K) {
    return K.reduce((Acc, K) => {
        return K in T ? { ...Acc, [K]: T[K] } : Acc;
    }, {});
}
export function PickResolve(T, K) {
    return (IsIntersectType(T) ? Intersect(FromIntersect(T.allOf, K)) :
        IsUnionType(T) ? Union(FromUnion(T.anyOf, K)) :
            IsObjectType(T) ? Object(FromProperties(T.properties, K)) :
                Object({}));
}
export function Pick(T, K, options = {}) {
    const I = IsSchemaType(K) ? IndexedKeyResolve(K) : K;
    const D = Discard(T, [TransformKind, '$id', 'required']);
    const R = CloneType(PickResolve(T, I), options);
    return { ...D, ...R };
}
