import { Object } from '../object/index.mjs';
import { Never } from '../never/index.mjs';
import { Union } from '../union/index.mjs';
import { IsTemplateLiteralFinite, TemplateLiteralParseExact } from '../template-literal/index.mjs';
import { PatternStringExact, PatternNumberExact } from '../patterns/index.mjs';
import { IndexedKeyResolve } from '../indexed/index.mjs';
import { Kind, Hint } from '../symbols/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { IsUndefined } from '../guard/value.mjs';
import { TUnion as IsUnionType, TTemplateLiteral as IsTemplateLiteralType, TLiteral as IsLiteralType, TString as IsStringType, TInteger as IsIntegerType, TNumber as IsNumberType } from '../guard/type.mjs';
function FromPattern(pattern, T) {
    return { [Kind]: 'Record', type: 'object', patternProperties: { [pattern]: CloneType(T) } };
}
function FromKeys(keys, T) {
    const properties = keys.reduce((acc, key) => ({ ...acc, [key]: CloneType(T) }), {});
    return Object(properties, { [Hint]: 'Record' });
}
function FromTemplateLiteralKey(K, T) {
    const expression = TemplateLiteralParseExact(K.pattern);
    return (IsTemplateLiteralFinite(expression)
        ? FromKeys(IndexedKeyResolve(K), T)
        : FromPattern(K.pattern, T));
}
function FromUnionKey(K, T) {
    return FromKeys(IndexedKeyResolve(Union(K)), T);
}
function FromLiteralKey(K, T) {
    return FromKeys([K.toString()], T);
}
function FromStringKey(K, T) {
    const pattern = IsUndefined(K.pattern) ? PatternStringExact : K.pattern;
    return FromPattern(pattern, T);
}
function FromIntegerKey(_, T) {
    return FromPattern(PatternNumberExact, T);
}
function FromNumberKey(_, T) {
    return FromPattern(PatternNumberExact, T);
}
export function RecordResolve(K, T) {
    return (IsUnionType(K) ? FromUnionKey(K.anyOf, T) :
        IsTemplateLiteralType(K) ? FromTemplateLiteralKey(K, T) :
            IsLiteralType(K) ? FromLiteralKey(K.const, T) :
                IsStringType(K) ? FromStringKey(K, T) :
                    IsIntegerType(K) ? FromIntegerKey(K, T) :
                        IsNumberType(K) ? FromNumberKey(K, T) :
                            Never());
}
export function Record(K, T, options = {}) {
    return CloneType(RecordResolve(K, T), options);
}
