import { PatternNumber, PatternString, PatternBoolean } from '../patterns/index.mjs';
import { Kind } from '../symbols/index.mjs';
import { TTemplateLiteral as IsTemplateLiteralType, TUnion as IsUnionType, TNumber as IsNumberType, TInteger as IsIntegerType, TBigInt as IsBigIntType, TString as IsStringType, TLiteral as IsLiteralType, TBoolean as IsBooleanType, } from '../guard/type.mjs';
export class TemplateLiteralPatternError extends Error {
}
function Escape(value) {
    return value.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
}
function Visit(schema, acc) {
    return (IsTemplateLiteralType(schema) ? schema.pattern.slice(1, schema.pattern.length - 1) :
        IsUnionType(schema) ? `(${schema.anyOf.map((schema) => Visit(schema, acc)).join('|')})` :
            IsNumberType(schema) ? `${acc}${PatternNumber}` :
                IsIntegerType(schema) ? `${acc}${PatternNumber}` :
                    IsBigIntType(schema) ? `${acc}${PatternNumber}` :
                        IsStringType(schema) ? `${acc}${PatternString}` :
                            IsLiteralType(schema) ? `${acc}${Escape(schema.const.toString())}` :
                                IsBooleanType(schema) ? `${acc}${PatternBoolean}` :
                                    (() => { throw new TemplateLiteralPatternError(`Unexpected Kind '${schema[Kind]}'`); })());
}
export function TemplateLiteralPattern(kinds) {
    return `^${kinds.map((schema) => Visit(schema, '')).join('')}\$`;
}
