export function IsAsyncIterator(value) {
    return IsObject(value) && Symbol.asyncIterator in value;
}
export function IsIterator(value) {
    return IsObject(value) && Symbol.iterator in value;
}
export function IsTypedArray(value) {
    return ArrayBuffer.isView(value);
}
export function IsPromise(value) {
    return value instanceof Promise;
}
export function IsUint8Array(value) {
    return value instanceof Uint8Array;
}
export function IsDate(value) {
    return value instanceof Date && Number.isFinite(value.getTime());
}
export function HasPropertyKey(value, key) {
    return key in value;
}
export function IsPlainObject(value) {
    return IsObject(value) && IsFunction(value.constructor) && value.constructor.name === 'Object';
}
export function IsObject(value) {
    return value !== null && typeof value === 'object';
}
export function IsArray(value) {
    return Array.isArray(value) && !ArrayBuffer.isView(value);
}
export function IsUndefined(value) {
    return value === undefined;
}
export function IsNull(value) {
    return value === null;
}
export function IsBoolean(value) {
    return typeof value === 'boolean';
}
export function IsNumber(value) {
    return typeof value === 'number';
}
export function IsInteger(value) {
    return IsNumber(value) && Number.isInteger(value);
}
export function IsBigInt(value) {
    return typeof value === 'bigint';
}
export function IsString(value) {
    return typeof value === 'string';
}
export function IsFunction(value) {
    return typeof value === 'function';
}
export function IsSymbol(value) {
    return typeof value === 'symbol';
}
export function IsValueType(value) {
    return (IsBigInt(value) ||
        IsBoolean(value) ||
        IsNull(value) ||
        IsNumber(value) ||
        IsString(value) ||
        IsSymbol(value) ||
        IsUndefined(value));
}
