import { TTransform as IsTransformType, TSchema as IsSchemaType } from '../../type/guard/type.mjs';
import { IsString, IsUndefined } from '../guard/index.mjs';
import { Deref } from '../deref/index.mjs';
import { Kind } from '../../type/symbols/index.mjs';
function TArray(schema, references) {
    return IsTransformType(schema) || Visit(schema.items, references);
}
function TAsyncIterator(schema, references) {
    return IsTransformType(schema) || Visit(schema.items, references);
}
function TConstructor(schema, references) {
    return IsTransformType(schema) || Visit(schema.returns, references) || schema.parameters.some((schema) => Visit(schema, references));
}
function TFunction(schema, references) {
    return IsTransformType(schema) || Visit(schema.returns, references) || schema.parameters.some((schema) => Visit(schema, references));
}
function TIntersect(schema, references) {
    return IsTransformType(schema) || IsTransformType(schema.unevaluatedProperties) || schema.allOf.some((schema) => Visit(schema, references));
}
function TIterator(schema, references) {
    return IsTransformType(schema) || Visit(schema.items, references);
}
function TNot(schema, references) {
    return IsTransformType(schema) || Visit(schema.not, references);
}
function TObject(schema, references) {
    return (IsTransformType(schema) ||
        Object.values(schema.properties).some((schema) => Visit(schema, references)) ||
        (IsSchemaType(schema.additionalProperties) && Visit(schema.additionalProperties, references)));
}
function TPromise(schema, references) {
    return IsTransformType(schema) || Visit(schema.item, references);
}
function TRecord(schema, references) {
    const pattern = Object.getOwnPropertyNames(schema.patternProperties)[0];
    const property = schema.patternProperties[pattern];
    return IsTransformType(schema) || Visit(property, references) || (IsSchemaType(schema.additionalProperties) && IsTransformType(schema.additionalProperties));
}
function TRef(schema, references) {
    if (IsTransformType(schema))
        return true;
    return Visit(Deref(schema, references), references);
}
function TThis(schema, references) {
    if (IsTransformType(schema))
        return true;
    return Visit(Deref(schema, references), references);
}
function TTuple(schema, references) {
    return IsTransformType(schema) || (!IsUndefined(schema.items) && schema.items.some((schema) => Visit(schema, references)));
}
function TUnion(schema, references) {
    return IsTransformType(schema) || schema.anyOf.some((schema) => Visit(schema, references));
}
function Visit(schema, references) {
    const references_ = IsString(schema.$id) ? [...references, schema] : references;
    const schema_ = schema;
    if (schema.$id && visited.has(schema.$id))
        return false;
    if (schema.$id)
        visited.add(schema.$id);
    switch (schema[Kind]) {
        case 'Array':
            return TArray(schema_, references_);
        case 'AsyncIterator':
            return TAsyncIterator(schema_, references_);
        case 'Constructor':
            return TConstructor(schema_, references_);
        case 'Function':
            return TFunction(schema_, references_);
        case 'Intersect':
            return TIntersect(schema_, references_);
        case 'Iterator':
            return TIterator(schema_, references_);
        case 'Not':
            return TNot(schema_, references_);
        case 'Object':
            return TObject(schema_, references_);
        case 'Promise':
            return TPromise(schema_, references_);
        case 'Record':
            return TRecord(schema_, references_);
        case 'Ref':
            return TRef(schema_, references_);
        case 'This':
            return TThis(schema_, references_);
        case 'Tuple':
            return TTuple(schema_, references_);
        case 'Union':
            return TUnion(schema_, references_);
        default:
            return IsTransformType(schema);
    }
}
const visited = new Set();
export function HasTransform(schema, references) {
    visited.clear();
    return Visit(schema, references);
}
