import { Decode as DecodeValue, Encode as EncodeValue, TransformDecodeCheckError, TransformEncodeCheckError } from '../transform/index.mjs';
import { Mutate as MutateValue } from '../mutate/index.mjs';
import { Hash as HashValue } from '../hash/index.mjs';
import { Equal as EqualValue } from '../equal/index.mjs';
import { Cast as CastValue } from '../cast/index.mjs';
import { Clone as CloneValue } from '../clone/index.mjs';
import { Convert as ConvertValue } from '../convert/index.mjs';
import { Create as CreateValue } from '../create/index.mjs';
import { Clean as CleanValue } from '../clean/index.mjs';
import { Check as CheckValue } from '../check/index.mjs';
import { Default as DefaultValue } from '../default/index.mjs';
import { Diff as DiffValue, Patch as PatchValue } from '../delta/index.mjs';
import { Errors as ValueErrors } from '../../errors/index.mjs';
export function Cast(...args) {
    return CastValue.apply(CastValue, args);
}
export function Create(...args) {
    return CreateValue.apply(CreateValue, args);
}
export function Check(...args) {
    return CheckValue.apply(CheckValue, args);
}
export function Clean(...args) {
    return CleanValue.apply(CleanValue, args);
}
export function Convert(...args) {
    return ConvertValue.apply(ConvertValue, args);
}
export function Clone(value) {
    return CloneValue(value);
}
export function Decode(...args) {
    const [schema, references, value] = args.length === 3 ? [args[0], args[1], args[2]] : [args[0], [], args[1]];
    if (!Check(schema, references, value))
        throw new TransformDecodeCheckError(schema, value, Errors(schema, references, value).First());
    return DecodeValue(schema, references, value);
}
export function Default(...args) {
    return DefaultValue.apply(DefaultValue, args);
}
export function Encode(...args) {
    const [schema, references, value] = args.length === 3 ? [args[0], args[1], args[2]] : [args[0], [], args[1]];
    const encoded = EncodeValue(schema, references, value);
    if (!Check(schema, references, encoded))
        throw new TransformEncodeCheckError(schema, value, Errors(schema, references, value).First());
    return encoded;
}
export function Errors(...args) {
    return ValueErrors.apply(ValueErrors, args);
}
export function Equal(left, right) {
    return EqualValue(left, right);
}
export function Diff(current, next) {
    return DiffValue(current, next);
}
export function Hash(value) {
    return HashValue(value);
}
export function Patch(current, edits) {
    return PatchValue(current, edits);
}
export function Mutate(current, next) {
    MutateValue(current, next);
}
