import { ValueErrorIterator } from '../errors/index';
import type { TSchema } from '../type/schema/index';
import type { Static, StaticDecode, StaticEncode } from '../type/static/index';
export type CheckFunction = (value: unknown) => boolean;
export declare class TypeCheck<T extends TSchema> {
    private readonly schema;
    private readonly references;
    private readonly checkFunc;
    private readonly code;
    private readonly hasTransform;
    constructor(schema: T, references: TSchema[], checkFunc: CheckFunction, code: string);
    Code(): string;
    Errors(value: unknown): ValueErrorIterator;
    Check(value: unknown): value is Static<T>;
    Decode(value: unknown): StaticDecode<T>;
    Encode(value: unknown): StaticEncode<T>;
}
export declare class TypeCompilerUnknownTypeError extends Error {
    readonly schema: TSchema;
    constructor(schema: TSchema);
}
export declare class TypeCompilerTypeGuardError extends Error {
    readonly schema: TSchema;
    constructor(schema: TSchema);
}
export declare namespace Policy {
    function IsExactOptionalProperty(value: string, key: string, expression: string): string;
    function IsObjectLike(value: string): string;
    function IsRecordLike(value: string): string;
    function IsNumberLike(value: string): string;
    function IsVoidLike(value: string): string;
}
export type TypeCompilerLanguageOption = 'typescript' | 'javascript';
export interface TypeCompilerCodegenOptions {
    language?: TypeCompilerLanguageOption;
}
export declare namespace TypeCompiler {
    function Code<T extends TSchema>(schema: T, references: TSchema[], options?: TypeCompilerCodegenOptions): string;
    function Code<T extends TSchema>(schema: T, options?: TypeCompilerCodegenOptions): string;
    function Compile<T extends TSchema>(schema: T, references?: TSchema[]): TypeCheck<T>;
}
