"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Deref = exports.DerefResolve = void 0;
const type_1 = require("../clone/type");
const index_1 = require("../discard/index");
const value_1 = require("../guard/value");
const type_2 = require("../guard/type");
function FromRest(schema, references) {
    return schema.map((schema) => Deref(schema, references));
}
function FromProperties(properties, references) {
    return globalThis.Object.getOwnPropertyNames(properties).reduce((acc, key) => {
        return { ...acc, [key]: Deref(properties[key], references) };
    }, {});
}
function FromConstructor(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.parameters = FromRest(clone.parameters, references);
    clone.returns = Deref(clone.returns, references);
    return clone;
}
function FromFunction(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.parameters = FromRest(clone.parameters, references);
    clone.returns = Deref(clone.returns, references);
    return clone;
}
function FromIntersect(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.allOf = FromRest(clone.allOf, references);
    return clone;
}
function FromUnion(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.anyOf = FromRest(clone.anyOf, references);
    return clone;
}
function FromTuple(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    if ((0, value_1.IsUndefined)(clone.items))
        return clone;
    clone.items = FromRest(clone.items, references);
    return clone;
}
function FromArray(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
function FromObject(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.properties = FromProperties(clone.properties, references);
    return clone;
}
function FromPromise(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.item = Deref(clone.item, references);
    return clone;
}
function FromAsyncIterator(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
function FromIterator(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
function FromRef(schema, references) {
    const target = references.find(remote => remote.$id === schema.$ref);
    if (target === undefined)
        throw Error(`Unable to dereference schema with $id ${schema.$ref}`);
    const discard = (0, index_1.Discard)(target, ['$id']);
    return Deref(discard, references);
}
function DerefResolve(schema, references) {
    return ((0, type_2.TConstructor)(schema) ? FromConstructor(schema, references) :
        (0, type_2.TFunction)(schema) ? FromFunction(schema, references) :
            (0, type_2.TIntersect)(schema) ? FromIntersect(schema, references) :
                (0, type_2.TUnion)(schema) ? FromUnion(schema, references) :
                    (0, type_2.TTuple)(schema) ? FromTuple(schema, references) :
                        (0, type_2.TArray)(schema) ? FromArray(schema, references) :
                            (0, type_2.TObject)(schema) ? FromObject(schema, references) :
                                (0, type_2.TPromise)(schema) ? FromPromise(schema, references) :
                                    (0, type_2.TAsyncIterator)(schema) ? FromAsyncIterator(schema, references) :
                                        (0, type_2.TIterator)(schema) ? FromIterator(schema, references) :
                                            (0, type_2.TRef)(schema) ? FromRef(schema, references) :
                                                schema);
}
exports.DerefResolve = DerefResolve;
function Deref(schema, references) {
    return DerefResolve(schema, references);
}
exports.Deref = Deref;
