"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtendsCheck = exports.ExtendsResult = exports.ExtendsResolverError = void 0;
const index_1 = require("../any/index");
const index_2 = require("../function/index");
const index_3 = require("../number/index");
const index_4 = require("../string/index");
const index_5 = require("../unknown/index");
const index_6 = require("../template-literal/index");
const index_7 = require("../patterns/index");
const index_8 = require("../symbols/index");
const index_9 = require("../guard/index");
class ExtendsResolverError extends Error {
}
exports.ExtendsResolverError = ExtendsResolverError;
var ExtendsResult;
(function (ExtendsResult) {
    ExtendsResult[ExtendsResult["Union"] = 0] = "Union";
    ExtendsResult[ExtendsResult["True"] = 1] = "True";
    ExtendsResult[ExtendsResult["False"] = 2] = "False";
})(ExtendsResult || (exports.ExtendsResult = ExtendsResult = {}));
function IntoBooleanResult(result) {
    return result === ExtendsResult.False ? result : ExtendsResult.True;
}
function Throw(message) {
    throw new ExtendsResolverError(message);
}
function IsStructuralRight(right) {
    return (index_9.TypeGuard.TNever(right) ||
        index_9.TypeGuard.TIntersect(right) ||
        index_9.TypeGuard.TUnion(right) ||
        index_9.TypeGuard.TUnknown(right) ||
        index_9.TypeGuard.TAny(right));
}
function StructuralRight(left, right) {
    return (index_9.TypeGuard.TNever(right) ? TNeverRight(left, right) :
        index_9.TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
            index_9.TypeGuard.TUnion(right) ? TUnionRight(left, right) :
                index_9.TypeGuard.TUnknown(right) ? TUnknownRight(left, right) :
                    index_9.TypeGuard.TAny(right) ? TAnyRight(left, right) :
                        Throw('StructuralRight'));
}
function TAnyRight(left, right) {
    return ExtendsResult.True;
}
function TAny(left, right) {
    return (index_9.TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
        (index_9.TypeGuard.TUnion(right) && right.anyOf.some((schema) => index_9.TypeGuard.TAny(schema) || index_9.TypeGuard.TUnknown(schema))) ? ExtendsResult.True :
            index_9.TypeGuard.TUnion(right) ? ExtendsResult.Union :
                index_9.TypeGuard.TUnknown(right) ? ExtendsResult.True :
                    index_9.TypeGuard.TAny(right) ? ExtendsResult.True :
                        ExtendsResult.Union);
}
function TArrayRight(left, right) {
    return (index_9.TypeGuard.TUnknown(left) ? ExtendsResult.False :
        index_9.TypeGuard.TAny(left) ? ExtendsResult.Union :
            index_9.TypeGuard.TNever(left) ? ExtendsResult.True :
                ExtendsResult.False);
}
function TArray(left, right) {
    return (index_9.TypeGuard.TObject(right) && IsObjectArrayLike(right) ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            !index_9.TypeGuard.TArray(right) ? ExtendsResult.False :
                IntoBooleanResult(Visit(left.items, right.items)));
}
function TAsyncIterator(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        !index_9.TypeGuard.TAsyncIterator(right) ? ExtendsResult.False :
            IntoBooleanResult(Visit(left.items, right.items)));
}
function TBigInt(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TBigInt(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TBooleanRight(left, right) {
    return (index_9.TypeGuard.TLiteralBoolean(left) ? ExtendsResult.True :
        index_9.TypeGuard.TBoolean(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TBoolean(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TBoolean(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TConstructor(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            !index_9.TypeGuard.TConstructor(right) ? ExtendsResult.False :
                left.parameters.length > right.parameters.length ? ExtendsResult.False :
                    (!left.parameters.every((schema, index) => IntoBooleanResult(Visit(right.parameters[index], schema)) === ExtendsResult.True)) ? ExtendsResult.False :
                        IntoBooleanResult(Visit(left.returns, right.returns)));
}
function TDate(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TDate(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TFunction(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            !index_9.TypeGuard.TFunction(right) ? ExtendsResult.False :
                left.parameters.length > right.parameters.length ? ExtendsResult.False :
                    (!left.parameters.every((schema, index) => IntoBooleanResult(Visit(right.parameters[index], schema)) === ExtendsResult.True)) ? ExtendsResult.False :
                        IntoBooleanResult(Visit(left.returns, right.returns)));
}
function TIntegerRight(left, right) {
    return (index_9.TypeGuard.TLiteral(left) && index_9.ValueGuard.IsNumber(left.const) ? ExtendsResult.True :
        index_9.TypeGuard.TNumber(left) || index_9.TypeGuard.TInteger(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TInteger(left, right) {
    return (index_9.TypeGuard.TInteger(right) || index_9.TypeGuard.TNumber(right) ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
                index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                    ExtendsResult.False);
}
function TIntersectRight(left, right) {
    return right.allOf.every((schema) => Visit(left, schema) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TIntersect(left, right) {
    return left.allOf.some((schema) => Visit(schema, right) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TIterator(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        !index_9.TypeGuard.TIterator(right) ? ExtendsResult.False :
            IntoBooleanResult(Visit(left.items, right.items)));
}
function TLiteral(left, right) {
    return (index_9.TypeGuard.TLiteral(right) && right.const === left.const ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
                index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                    index_9.TypeGuard.TString(right) ? TStringRight(left, right) :
                        index_9.TypeGuard.TNumber(right) ? TNumberRight(left, right) :
                            index_9.TypeGuard.TInteger(right) ? TIntegerRight(left, right) :
                                index_9.TypeGuard.TBoolean(right) ? TBooleanRight(left, right) :
                                    ExtendsResult.False);
}
function TNeverRight(left, right) {
    return ExtendsResult.False;
}
function TNever(left, right) {
    return ExtendsResult.True;
}
function UnwrapTNot(schema) {
    let [current, depth] = [schema, 0];
    while (true) {
        if (!index_9.TypeGuard.TNot(current))
            break;
        current = current.not;
        depth += 1;
    }
    return depth % 2 === 0 ? current : (0, index_5.Unknown)();
}
function TNot(left, right) {
    return (index_9.TypeGuard.TNot(left) ? Visit(UnwrapTNot(left), right) :
        index_9.TypeGuard.TNot(right) ? Visit(left, UnwrapTNot(right)) :
            Throw('Invalid fallthrough for Not'));
}
function TNull(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TNull(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TNumberRight(left, right) {
    return (index_9.TypeGuard.TLiteralNumber(left) ? ExtendsResult.True :
        index_9.TypeGuard.TNumber(left) || index_9.TypeGuard.TInteger(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TNumber(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TInteger(right) || index_9.TypeGuard.TNumber(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function IsObjectPropertyCount(schema, count) {
    return Object.getOwnPropertyNames(schema.properties).length === count;
}
function IsObjectStringLike(schema) {
    return IsObjectArrayLike(schema);
}
function IsObjectSymbolLike(schema) {
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'description' in schema.properties && index_9.TypeGuard.TUnion(schema.properties.description) && schema.properties.description.anyOf.length === 2 && ((index_9.TypeGuard.TString(schema.properties.description.anyOf[0]) &&
        index_9.TypeGuard.TUndefined(schema.properties.description.anyOf[1])) || (index_9.TypeGuard.TString(schema.properties.description.anyOf[1]) &&
        index_9.TypeGuard.TUndefined(schema.properties.description.anyOf[0]))));
}
function IsObjectNumberLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectBooleanLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectBigIntLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectDateLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectUint8ArrayLike(schema) {
    return IsObjectArrayLike(schema);
}
function IsObjectFunctionLike(schema) {
    const length = (0, index_3.Number)();
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'length' in schema.properties && IntoBooleanResult(Visit(schema.properties['length'], length)) === ExtendsResult.True);
}
function IsObjectConstructorLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
function IsObjectArrayLike(schema) {
    const length = (0, index_3.Number)();
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'length' in schema.properties && IntoBooleanResult(Visit(schema.properties['length'], length)) === ExtendsResult.True);
}
function IsObjectPromiseLike(schema) {
    const then = (0, index_2.Function)([(0, index_1.Any)()], (0, index_1.Any)());
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'then' in schema.properties && IntoBooleanResult(Visit(schema.properties['then'], then)) === ExtendsResult.True);
}
function Property(left, right) {
    return (Visit(left, right) === ExtendsResult.False ? ExtendsResult.False :
        index_9.TypeGuard.TOptional(left) && !index_9.TypeGuard.TOptional(right) ? ExtendsResult.False :
            ExtendsResult.True);
}
function TObjectRight(left, right) {
    return (index_9.TypeGuard.TUnknown(left) ? ExtendsResult.False :
        index_9.TypeGuard.TAny(left) ? ExtendsResult.Union : (index_9.TypeGuard.TNever(left) ||
            (index_9.TypeGuard.TLiteralString(left) && IsObjectStringLike(right)) ||
            (index_9.TypeGuard.TLiteralNumber(left) && IsObjectNumberLike(right)) ||
            (index_9.TypeGuard.TLiteralBoolean(left) && IsObjectBooleanLike(right)) ||
            (index_9.TypeGuard.TSymbol(left) && IsObjectSymbolLike(right)) ||
            (index_9.TypeGuard.TBigInt(left) && IsObjectBigIntLike(right)) ||
            (index_9.TypeGuard.TString(left) && IsObjectStringLike(right)) ||
            (index_9.TypeGuard.TSymbol(left) && IsObjectSymbolLike(right)) ||
            (index_9.TypeGuard.TNumber(left) && IsObjectNumberLike(right)) ||
            (index_9.TypeGuard.TInteger(left) && IsObjectNumberLike(right)) ||
            (index_9.TypeGuard.TBoolean(left) && IsObjectBooleanLike(right)) ||
            (index_9.TypeGuard.TUint8Array(left) && IsObjectUint8ArrayLike(right)) ||
            (index_9.TypeGuard.TDate(left) && IsObjectDateLike(right)) ||
            (index_9.TypeGuard.TConstructor(left) && IsObjectConstructorLike(right)) ||
            (index_9.TypeGuard.TFunction(left) && IsObjectFunctionLike(right))) ? ExtendsResult.True :
            (index_9.TypeGuard.TRecord(left) && index_9.TypeGuard.TString(RecordKey(left))) ? (() => {
                return right[index_8.Hint] === 'Record' ? ExtendsResult.True : ExtendsResult.False;
            })() :
                (index_9.TypeGuard.TRecord(left) && index_9.TypeGuard.TNumber(RecordKey(left))) ? (() => {
                    return IsObjectPropertyCount(right, 0) ? ExtendsResult.True : ExtendsResult.False;
                })() :
                    ExtendsResult.False);
}
function TObject(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
            !index_9.TypeGuard.TObject(right) ? ExtendsResult.False :
                (() => {
                    for (const key of Object.getOwnPropertyNames(right.properties)) {
                        if (!(key in left.properties) && !index_9.TypeGuard.TOptional(right.properties[key])) {
                            return ExtendsResult.False;
                        }
                        if (index_9.TypeGuard.TOptional(right.properties[key])) {
                            return ExtendsResult.True;
                        }
                        if (Property(left.properties[key], right.properties[key]) === ExtendsResult.False) {
                            return ExtendsResult.False;
                        }
                    }
                    return ExtendsResult.True;
                })());
}
function TPromise(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) && IsObjectPromiseLike(right) ? ExtendsResult.True :
            !index_9.TypeGuard.TPromise(right) ? ExtendsResult.False :
                IntoBooleanResult(Visit(left.item, right.item)));
}
function RecordKey(schema) {
    return (index_7.PatternNumberExact in schema.patternProperties ? (0, index_3.Number)() :
        index_7.PatternStringExact in schema.patternProperties ? (0, index_4.String)() :
            Throw('Unknown record key pattern'));
}
function RecordValue(schema) {
    return (index_7.PatternNumberExact in schema.patternProperties ? schema.patternProperties[index_7.PatternNumberExact] :
        index_7.PatternStringExact in schema.patternProperties ? schema.patternProperties[index_7.PatternStringExact] :
            Throw('Unable to get record value schema'));
}
function TRecordRight(left, right) {
    const [Key, Value] = [RecordKey(right), RecordValue(right)];
    return ((index_9.TypeGuard.TLiteralString(left) && index_9.TypeGuard.TNumber(Key) && IntoBooleanResult(Visit(left, Value)) === ExtendsResult.True) ? ExtendsResult.True :
        index_9.TypeGuard.TUint8Array(left) && index_9.TypeGuard.TNumber(Key) ? Visit(left, Value) :
            index_9.TypeGuard.TString(left) && index_9.TypeGuard.TNumber(Key) ? Visit(left, Value) :
                index_9.TypeGuard.TArray(left) && index_9.TypeGuard.TNumber(Key) ? Visit(left, Value) :
                    index_9.TypeGuard.TObject(left) ? (() => {
                        for (const key of Object.getOwnPropertyNames(left.properties)) {
                            if (Property(Value, left.properties[key]) === ExtendsResult.False) {
                                return ExtendsResult.False;
                            }
                        }
                        return ExtendsResult.True;
                    })() :
                        ExtendsResult.False);
}
function TRecord(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            !index_9.TypeGuard.TRecord(right) ? ExtendsResult.False :
                Visit(RecordValue(left), RecordValue(right)));
}
function TStringRight(left, right) {
    return (index_9.TypeGuard.TLiteral(left) && index_9.ValueGuard.IsString(left.const) ? ExtendsResult.True :
        index_9.TypeGuard.TString(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TString(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TString(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TSymbol(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TSymbol(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TTemplateLiteral(left, right) {
    return (index_9.TypeGuard.TTemplateLiteral(left) ? Visit((0, index_6.TemplateLiteralToUnion)(left), right) :
        index_9.TypeGuard.TTemplateLiteral(right) ? Visit(left, (0, index_6.TemplateLiteralToUnion)(right)) :
            Throw('Invalid fallthrough for TemplateLiteral'));
}
function IsArrayOfTuple(left, right) {
    return (index_9.TypeGuard.TArray(right) &&
        left.items !== undefined &&
        left.items.every((schema) => Visit(schema, right.items) === ExtendsResult.True));
}
function TTupleRight(left, right) {
    return (index_9.TypeGuard.TNever(left) ? ExtendsResult.True :
        index_9.TypeGuard.TUnknown(left) ? ExtendsResult.False :
            index_9.TypeGuard.TAny(left) ? ExtendsResult.Union :
                ExtendsResult.False);
}
function TTuple(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) && IsObjectArrayLike(right) ? ExtendsResult.True :
            index_9.TypeGuard.TArray(right) && IsArrayOfTuple(left, right) ? ExtendsResult.True :
                !index_9.TypeGuard.TTuple(right) ? ExtendsResult.False :
                    (index_9.ValueGuard.IsUndefined(left.items) && !index_9.ValueGuard.IsUndefined(right.items)) || (!index_9.ValueGuard.IsUndefined(left.items) && index_9.ValueGuard.IsUndefined(right.items)) ? ExtendsResult.False :
                        (index_9.ValueGuard.IsUndefined(left.items) && !index_9.ValueGuard.IsUndefined(right.items)) ? ExtendsResult.True :
                            left.items.every((schema, index) => Visit(schema, right.items[index]) === ExtendsResult.True) ? ExtendsResult.True :
                                ExtendsResult.False);
}
function TUint8Array(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TUint8Array(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
function TUndefined(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
            index_9.TypeGuard.TRecord(right) ? TRecordRight(left, right) :
                index_9.TypeGuard.TVoid(right) ? VoidRight(left, right) :
                    index_9.TypeGuard.TUndefined(right) ? ExtendsResult.True :
                        ExtendsResult.False);
}
function TUnionRight(left, right) {
    return right.anyOf.some((schema) => Visit(left, schema) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TUnion(left, right) {
    return left.anyOf.every((schema) => Visit(schema, right) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
function TUnknownRight(left, right) {
    return ExtendsResult.True;
}
function TUnknown(left, right) {
    return (index_9.TypeGuard.TNever(right) ? TNeverRight(left, right) :
        index_9.TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
            index_9.TypeGuard.TUnion(right) ? TUnionRight(left, right) :
                index_9.TypeGuard.TAny(right) ? TAnyRight(left, right) :
                    index_9.TypeGuard.TString(right) ? TStringRight(left, right) :
                        index_9.TypeGuard.TNumber(right) ? TNumberRight(left, right) :
                            index_9.TypeGuard.TInteger(right) ? TIntegerRight(left, right) :
                                index_9.TypeGuard.TBoolean(right) ? TBooleanRight(left, right) :
                                    index_9.TypeGuard.TArray(right) ? TArrayRight(left, right) :
                                        index_9.TypeGuard.TTuple(right) ? TTupleRight(left, right) :
                                            index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
                                                index_9.TypeGuard.TUnknown(right) ? ExtendsResult.True :
                                                    ExtendsResult.False);
}
function VoidRight(left, right) {
    return (index_9.TypeGuard.TUndefined(left) ? ExtendsResult.True :
        index_9.TypeGuard.TUndefined(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
function TVoid(left, right) {
    return (index_9.TypeGuard.TIntersect(right) ? TIntersectRight(left, right) :
        index_9.TypeGuard.TUnion(right) ? TUnionRight(left, right) :
            index_9.TypeGuard.TUnknown(right) ? TUnknownRight(left, right) :
                index_9.TypeGuard.TAny(right) ? TAnyRight(left, right) :
                    index_9.TypeGuard.TObject(right) ? TObjectRight(left, right) :
                        index_9.TypeGuard.TVoid(right) ? ExtendsResult.True :
                            ExtendsResult.False);
}
function Visit(left, right) {
    return ((index_9.TypeGuard.TTemplateLiteral(left) || index_9.TypeGuard.TTemplateLiteral(right)) ? TTemplateLiteral(left, right) :
        (index_9.TypeGuard.TNot(left) || index_9.TypeGuard.TNot(right)) ? TNot(left, right) :
            index_9.TypeGuard.TAny(left) ? TAny(left, right) :
                index_9.TypeGuard.TArray(left) ? TArray(left, right) :
                    index_9.TypeGuard.TBigInt(left) ? TBigInt(left, right) :
                        index_9.TypeGuard.TBoolean(left) ? TBoolean(left, right) :
                            index_9.TypeGuard.TAsyncIterator(left) ? TAsyncIterator(left, right) :
                                index_9.TypeGuard.TConstructor(left) ? TConstructor(left, right) :
                                    index_9.TypeGuard.TDate(left) ? TDate(left, right) :
                                        index_9.TypeGuard.TFunction(left) ? TFunction(left, right) :
                                            index_9.TypeGuard.TInteger(left) ? TInteger(left, right) :
                                                index_9.TypeGuard.TIntersect(left) ? TIntersect(left, right) :
                                                    index_9.TypeGuard.TIterator(left) ? TIterator(left, right) :
                                                        index_9.TypeGuard.TLiteral(left) ? TLiteral(left, right) :
                                                            index_9.TypeGuard.TNever(left) ? TNever(left, right) :
                                                                index_9.TypeGuard.TNull(left) ? TNull(left, right) :
                                                                    index_9.TypeGuard.TNumber(left) ? TNumber(left, right) :
                                                                        index_9.TypeGuard.TObject(left) ? TObject(left, right) :
                                                                            index_9.TypeGuard.TRecord(left) ? TRecord(left, right) :
                                                                                index_9.TypeGuard.TString(left) ? TString(left, right) :
                                                                                    index_9.TypeGuard.TSymbol(left) ? TSymbol(left, right) :
                                                                                        index_9.TypeGuard.TTuple(left) ? TTuple(left, right) :
                                                                                            index_9.TypeGuard.TPromise(left) ? TPromise(left, right) :
                                                                                                index_9.TypeGuard.TUint8Array(left) ? TUint8Array(left, right) :
                                                                                                    index_9.TypeGuard.TUndefined(left) ? TUndefined(left, right) :
                                                                                                        index_9.TypeGuard.TUnion(left) ? TUnion(left, right) :
                                                                                                            index_9.TypeGuard.TUnknown(left) ? TUnknown(left, right) :
                                                                                                                index_9.TypeGuard.TVoid(left) ? TVoid(left, right) :
                                                                                                                    Throw(`Unknown left type operand '${left[index_8.Kind]}'`));
}
function ExtendsCheck(left, right) {
    return Visit(left, right);
}
exports.ExtendsCheck = ExtendsCheck;
