"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSchema = exports.TKind = exports.TVoid = exports.TUnsafe = exports.TUnknown = exports.TUint8Array = exports.TUnion = exports.TUnionLiteral = exports.TUndefined = exports.TTuple = exports.TTransform = exports.TThis = exports.TTemplateLiteral = exports.TSymbol = exports.TString = exports.TRef = exports.TRecursive = exports.TRecord = exports.TPromise = exports.TObject = exports.TNumber = exports.TNull = exports.TNot = exports.TNever = exports.TLiteral = exports.TLiteralBoolean = exports.TLiteralNumber = exports.TLiteralString = exports.TKindOf = exports.TIterator = exports.TIntersect = exports.TInteger = exports.TFunction = exports.TDate = exports.TConstructor = exports.TBoolean = exports.TBigInt = exports.TAsyncIterator = exports.TArray = exports.TAny = exports.TOptional = exports.TReadonly = exports.TypeGuardUnknownTypeError = void 0;
const ValueGuard = require("./value");
const index_1 = require("../symbols/index");
class TypeGuardUnknownTypeError extends Error {
}
exports.TypeGuardUnknownTypeError = TypeGuardUnknownTypeError;
const KnownTypes = [
    'Any',
    'Array',
    'AsyncIterator',
    'BigInt',
    'Boolean',
    'Constructor',
    'Date',
    'Enum',
    'Function',
    'Integer',
    'Intersect',
    'Iterator',
    'Literal',
    'Not',
    'Null',
    'Number',
    'Object',
    'Promise',
    'Record',
    'Ref',
    'String',
    'Symbol',
    'TemplateLiteral',
    'This',
    'Tuple',
    'Undefined',
    'Union',
    'Uint8Array',
    'Unknown',
    'Void',
];
function IsPattern(value) {
    try {
        new RegExp(value);
        return true;
    }
    catch {
        return false;
    }
}
function IsControlCharacterFree(value) {
    if (!ValueGuard.IsString(value))
        return false;
    for (let i = 0; i < value.length; i++) {
        const code = value.charCodeAt(i);
        if ((code >= 7 && code <= 13) || code === 27 || code === 127) {
            return false;
        }
    }
    return true;
}
function IsAdditionalProperties(value) {
    return IsOptionalBoolean(value) || (0, index_2.TSchema)(value);
}
function IsOptionalBigInt(value) {
    return ValueGuard.IsUndefined(value) || ValueGuard.IsBigInt(value);
}
function IsOptionalNumber(value) {
    return ValueGuard.IsUndefined(value) || ValueGuard.IsNumber(value);
}
function IsOptionalBoolean(value) {
    return ValueGuard.IsUndefined(value) || ValueGuard.IsBoolean(value);
}
function IsOptionalString(value) {
    return ValueGuard.IsUndefined(value) || ValueGuard.IsString(value);
}
function IsOptionalPattern(value) {
    return ValueGuard.IsUndefined(value) || (ValueGuard.IsString(value) && IsControlCharacterFree(value) && IsPattern(value));
}
function IsOptionalFormat(value) {
    return ValueGuard.IsUndefined(value) || (ValueGuard.IsString(value) && IsControlCharacterFree(value));
}
function IsOptionalSchema(value) {
    return ValueGuard.IsUndefined(value) || (0, index_2.TSchema)(value);
}
function TReadonly(schema) {
    return ValueGuard.IsObject(schema) && schema[index_1.ReadonlyKind] === 'Readonly';
}
exports.TReadonly = TReadonly;
function TOptional(schema) {
    return ValueGuard.IsObject(schema) && schema[index_1.OptionalKind] === 'Optional';
}
exports.TOptional = TOptional;
function TAny(schema) {
    return (TKindOf(schema, 'Any') &&
        IsOptionalString(schema.$id));
}
exports.TAny = TAny;
function TArray(schema) {
    return (TKindOf(schema, 'Array') &&
        schema.type === 'array' &&
        IsOptionalString(schema.$id) &&
        (0, index_2.TSchema)(schema.items) &&
        IsOptionalNumber(schema.minItems) &&
        IsOptionalNumber(schema.maxItems) &&
        IsOptionalBoolean(schema.uniqueItems) &&
        IsOptionalSchema(schema.contains) &&
        IsOptionalNumber(schema.minContains) &&
        IsOptionalNumber(schema.maxContains));
}
exports.TArray = TArray;
function TAsyncIterator(schema) {
    return (TKindOf(schema, 'AsyncIterator') &&
        schema.type === 'AsyncIterator' &&
        IsOptionalString(schema.$id) &&
        (0, index_2.TSchema)(schema.items));
}
exports.TAsyncIterator = TAsyncIterator;
function TBigInt(schema) {
    return (TKindOf(schema, 'BigInt') &&
        schema.type === 'bigint' &&
        IsOptionalString(schema.$id) &&
        IsOptionalBigInt(schema.exclusiveMaximum) &&
        IsOptionalBigInt(schema.exclusiveMinimum) &&
        IsOptionalBigInt(schema.maximum) &&
        IsOptionalBigInt(schema.minimum) &&
        IsOptionalBigInt(schema.multipleOf));
}
exports.TBigInt = TBigInt;
function TBoolean(schema) {
    return (TKindOf(schema, 'Boolean') &&
        schema.type === 'boolean' &&
        IsOptionalString(schema.$id));
}
exports.TBoolean = TBoolean;
function TConstructor(schema) {
    return (TKindOf(schema, 'Constructor') &&
        schema.type === 'Constructor' &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsArray(schema.parameters) &&
        schema.parameters.every(schema => (0, index_2.TSchema)(schema)) &&
        (0, index_2.TSchema)(schema.returns));
}
exports.TConstructor = TConstructor;
function TDate(schema) {
    return (TKindOf(schema, 'Date') &&
        schema.type === 'Date' &&
        IsOptionalString(schema.$id) &&
        IsOptionalNumber(schema.exclusiveMaximumTimestamp) &&
        IsOptionalNumber(schema.exclusiveMinimumTimestamp) &&
        IsOptionalNumber(schema.maximumTimestamp) &&
        IsOptionalNumber(schema.minimumTimestamp) &&
        IsOptionalNumber(schema.multipleOfTimestamp));
}
exports.TDate = TDate;
function TFunction(schema) {
    return (TKindOf(schema, 'Function') &&
        schema.type === 'Function' &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsArray(schema.parameters) &&
        schema.parameters.every(schema => (0, index_2.TSchema)(schema)) &&
        (0, index_2.TSchema)(schema.returns));
}
exports.TFunction = TFunction;
function TInteger(schema) {
    return (TKindOf(schema, 'Integer') &&
        schema.type === 'integer' &&
        IsOptionalString(schema.$id) &&
        IsOptionalNumber(schema.exclusiveMaximum) &&
        IsOptionalNumber(schema.exclusiveMinimum) &&
        IsOptionalNumber(schema.maximum) &&
        IsOptionalNumber(schema.minimum) &&
        IsOptionalNumber(schema.multipleOf));
}
exports.TInteger = TInteger;
function TIntersect(schema) {
    return (TKindOf(schema, 'Intersect') &&
        (ValueGuard.IsString(schema.type) && schema.type !== 'object' ? false : true) &&
        ValueGuard.IsArray(schema.allOf) &&
        schema.allOf.every(schema => (0, index_2.TSchema)(schema) && !TTransform(schema)) &&
        IsOptionalString(schema.type) &&
        (IsOptionalBoolean(schema.unevaluatedProperties) || IsOptionalSchema(schema.unevaluatedProperties)) &&
        IsOptionalString(schema.$id));
}
exports.TIntersect = TIntersect;
function TIterator(schema) {
    return (TKindOf(schema, 'Iterator') &&
        schema.type === 'Iterator' &&
        IsOptionalString(schema.$id) &&
        (0, index_2.TSchema)(schema.items));
}
exports.TIterator = TIterator;
function TKindOf(schema, kind) {
    return ValueGuard.IsObject(schema) && index_1.Kind in schema && schema[index_1.Kind] === kind;
}
exports.TKindOf = TKindOf;
function TLiteralString(schema) {
    return (0, index_3.TLiteral)(schema) && ValueGuard.IsString(schema.const);
}
exports.TLiteralString = TLiteralString;
function TLiteralNumber(schema) {
    return (0, index_3.TLiteral)(schema) && ValueGuard.IsNumber(schema.const);
}
exports.TLiteralNumber = TLiteralNumber;
function TLiteralBoolean(schema) {
    return (0, index_3.TLiteral)(schema) && ValueGuard.IsBoolean(schema.const);
}
exports.TLiteralBoolean = TLiteralBoolean;
function TLiteral(schema) {
    return (TKindOf(schema, 'Literal') &&
        IsOptionalString(schema.$id) && (ValueGuard.IsBoolean(schema.const) ||
        ValueGuard.IsNumber(schema.const) ||
        ValueGuard.IsString(schema.const)));
}
exports.TLiteral = TLiteral;
function TNever(schema) {
    return (TKindOf(schema, 'Never') &&
        ValueGuard.IsObject(schema.not) &&
        Object.getOwnPropertyNames(schema.not).length === 0);
}
exports.TNever = TNever;
function TNot(schema) {
    return (TKindOf(schema, 'Not') &&
        (0, index_2.TSchema)(schema.not));
}
exports.TNot = TNot;
function TNull(schema) {
    return (TKindOf(schema, 'Null') &&
        schema.type === 'null' &&
        IsOptionalString(schema.$id));
}
exports.TNull = TNull;
function TNumber(schema) {
    return (TKindOf(schema, 'Number') &&
        schema.type === 'number' &&
        IsOptionalString(schema.$id) &&
        IsOptionalNumber(schema.exclusiveMaximum) &&
        IsOptionalNumber(schema.exclusiveMinimum) &&
        IsOptionalNumber(schema.maximum) &&
        IsOptionalNumber(schema.minimum) &&
        IsOptionalNumber(schema.multipleOf));
}
exports.TNumber = TNumber;
function TObject(schema) {
    return (TKindOf(schema, 'Object') &&
        schema.type === 'object' &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsObject(schema.properties) &&
        Object.entries(schema.properties).every(([key, schema]) => IsControlCharacterFree(key) && (0, index_2.TSchema)(schema)) &&
        IsAdditionalProperties(schema.additionalProperties) &&
        IsOptionalNumber(schema.minProperties) &&
        IsOptionalNumber(schema.maxProperties));
}
exports.TObject = TObject;
function TPromise(schema) {
    return (TKindOf(schema, 'Promise') &&
        schema.type === 'Promise' &&
        IsOptionalString(schema.$id) &&
        (0, index_2.TSchema)(schema.item));
}
exports.TPromise = TPromise;
function TRecord(schema) {
    return (TKindOf(schema, 'Record') &&
        schema.type === 'object' &&
        IsOptionalString(schema.$id) &&
        IsAdditionalProperties(schema.additionalProperties) &&
        ValueGuard.IsObject(schema.patternProperties) &&
        ((schema) => {
            const keys = Object.getOwnPropertyNames(schema.patternProperties);
            return (keys.length === 1 &&
                IsPattern(keys[0]) &&
                ValueGuard.IsObject(schema.patternProperties) &&
                (0, index_2.TSchema)(schema.patternProperties[keys[0]]));
        })(schema));
}
exports.TRecord = TRecord;
function TRecursive(schema) {
    return ValueGuard.IsObject(schema) && index_1.Hint in schema && schema[index_1.Hint] === 'Recursive';
}
exports.TRecursive = TRecursive;
function TRef(schema) {
    return (TKindOf(schema, 'Ref') &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsString(schema.$ref));
}
exports.TRef = TRef;
function TString(schema) {
    return (TKindOf(schema, 'String') &&
        schema.type === 'string' &&
        IsOptionalString(schema.$id) &&
        IsOptionalNumber(schema.minLength) &&
        IsOptionalNumber(schema.maxLength) &&
        IsOptionalPattern(schema.pattern) &&
        IsOptionalFormat(schema.format));
}
exports.TString = TString;
function TSymbol(schema) {
    return (TKindOf(schema, 'Symbol') &&
        schema.type === 'symbol' &&
        IsOptionalString(schema.$id));
}
exports.TSymbol = TSymbol;
function TTemplateLiteral(schema) {
    return (TKindOf(schema, 'TemplateLiteral') &&
        schema.type === 'string' &&
        ValueGuard.IsString(schema.pattern) &&
        schema.pattern[0] === '^' &&
        schema.pattern[schema.pattern.length - 1] === '$');
}
exports.TTemplateLiteral = TTemplateLiteral;
function TThis(schema) {
    return (TKindOf(schema, 'This') &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsString(schema.$ref));
}
exports.TThis = TThis;
function TTransform(schema) {
    return ValueGuard.IsObject(schema) && index_1.TransformKind in schema;
}
exports.TTransform = TTransform;
function TTuple(schema) {
    return (TKindOf(schema, 'Tuple') &&
        schema.type === 'array' &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsNumber(schema.minItems) &&
        ValueGuard.IsNumber(schema.maxItems) &&
        schema.minItems === schema.maxItems &&
        ((ValueGuard.IsUndefined(schema.items) &&
            ValueGuard.IsUndefined(schema.additionalItems) &&
            schema.minItems === 0) || (ValueGuard.IsArray(schema.items) &&
            schema.items.every(schema => (0, index_2.TSchema)(schema)))));
}
exports.TTuple = TTuple;
function TUndefined(schema) {
    return (TKindOf(schema, 'Undefined') &&
        schema.type === 'undefined' &&
        IsOptionalString(schema.$id));
}
exports.TUndefined = TUndefined;
function TUnionLiteral(schema) {
    return (0, index_4.TUnion)(schema) && schema.anyOf.every((schema) => TLiteralString(schema) || TLiteralNumber(schema));
}
exports.TUnionLiteral = TUnionLiteral;
function TUnion(schema) {
    return (TKindOf(schema, 'Union') &&
        IsOptionalString(schema.$id) &&
        ValueGuard.IsObject(schema) &&
        ValueGuard.IsArray(schema.anyOf) &&
        schema.anyOf.every(schema => (0, index_2.TSchema)(schema)));
}
exports.TUnion = TUnion;
function TUint8Array(schema) {
    return (TKindOf(schema, 'Uint8Array') &&
        schema.type === 'Uint8Array' &&
        IsOptionalString(schema.$id) &&
        IsOptionalNumber(schema.minByteLength) &&
        IsOptionalNumber(schema.maxByteLength));
}
exports.TUint8Array = TUint8Array;
function TUnknown(schema) {
    return (TKindOf(schema, 'Unknown') &&
        IsOptionalString(schema.$id));
}
exports.TUnknown = TUnknown;
function TUnsafe(schema) {
    return TKindOf(schema, 'Unsafe');
}
exports.TUnsafe = TUnsafe;
function TVoid(schema) {
    return (TKindOf(schema, 'Void') &&
        schema.type === 'void' &&
        IsOptionalString(schema.$id));
}
exports.TVoid = TVoid;
function TKind(schema) {
    return ValueGuard.IsObject(schema) && index_1.Kind in schema && ValueGuard.IsString(schema[index_1.Kind]) && !KnownTypes.includes(schema[index_1.Kind]);
}
exports.TKind = TKind;
function TSchema(schema) {
    return (ValueGuard.IsObject(schema)) && ((0, index_5.TAny)(schema) ||
        (0, index_6.TArray)(schema) ||
        (0, index_7.TBoolean)(schema) ||
        (0, index_8.TBigInt)(schema) ||
        (0, index_9.TAsyncIterator)(schema) ||
        (0, index_10.TConstructor)(schema) ||
        (0, index_11.TDate)(schema) ||
        (0, index_12.TFunction)(schema) ||
        (0, index_13.TInteger)(schema) ||
        (0, index_14.TIntersect)(schema) ||
        (0, index_15.TIterator)(schema) ||
        (0, index_3.TLiteral)(schema) ||
        (0, index_16.TNever)(schema) ||
        (0, index_17.TNot)(schema) ||
        (0, index_18.TNull)(schema) ||
        (0, index_19.TNumber)(schema) ||
        (0, index_20.TObject)(schema) ||
        (0, index_21.TPromise)(schema) ||
        (0, index_22.TRecord)(schema) ||
        (0, index_23.TRef)(schema) ||
        (0, index_24.TString)(schema) ||
        (0, index_25.TSymbol)(schema) ||
        (0, index_26.TTemplateLiteral)(schema) ||
        (0, index_27.TThis)(schema) ||
        (0, index_28.TTuple)(schema) ||
        (0, index_29.TUndefined)(schema) ||
        (0, index_4.TUnion)(schema) ||
        (0, index_30.TUint8Array)(schema) ||
        (0, index_31.TUnknown)(schema) ||
        (0, index_32.TUnsafe)(schema) ||
        (0, index_33.TVoid)(schema) ||
        TKind(schema));
}
exports.TSchema = TSchema;
