"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexedTypeResolve = void 0;
const index_1 = require("../never/index");
const index_2 = require("../intersect/index");
const index_3 = require("../union/index");
const type_1 = require("../guard/type");
function FromRest(T, K) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [FromKey(L, K), ...FromRest(R, K)]
        : []);
}
function FromIntersectRest(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? (0, type_1.TNever)(L)
            ? [...FromIntersectRest(R)]
            : [L, ...FromIntersectRest(R)]
        : []);
}
function FromIntersect(T, K) {
    return ((0, index_2.IntersectResolve)(FromIntersectRest(FromRest(T, K))));
}
function FromUnionRest(T, S = T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? (0, type_1.TNever)(L)
            ? []
            : FromUnionRest(R, S)
        : S);
}
function FromUnion(T, K) {
    return ((0, index_3.UnionResolve)(FromUnionRest(FromRest(T, K))));
}
function FromTuple(T, K) {
    return (K in T ? T[K] :
        K === '[number]' ? (0, index_3.UnionResolve)(T) :
            (0, index_1.Never)());
}
function FromArray(T, K) {
    return (K === '[number]'
        ? T
        : (0, index_1.Never)());
}
function FromProperty(T, K) {
    return (K in T
        ? T[K]
        : (0, index_1.Never)());
}
function FromKey(T, K) {
    return ((0, type_1.TIntersect)(T) ? FromIntersect(T.allOf, K) :
        (0, type_1.TUnion)(T) ? FromUnion(T.anyOf, K) :
            (0, type_1.TTuple)(T) ? FromTuple(T.items ?? [], K) :
                (0, type_1.TArray)(T) ? FromArray(T.items, K) :
                    (0, type_1.TObject)(T) ? FromProperty(T.properties, K) :
                        (0, index_1.Never)());
}
function FromKeys(T, K) {
    const [L, ...R] = K;
    return (K.length > 0
        ? [FromKey(T, L), ...FromKeys(T, R)]
        : []);
}
function IndexedTypeResolve(T, K) {
    return ((0, index_3.UnionResolve)(FromKeys(T, K)));
}
exports.IndexedTypeResolve = IndexedTypeResolve;
