"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Capitalize = exports.Lowercase = exports.Uppercase = exports.Uncapitalize = exports.IntrinsicResolve = void 0;
const index_1 = require("../template-literal/index");
const index_2 = require("../literal/index");
const index_3 = require("../union/index");
const type_1 = require("../guard/type");
const type_2 = require("../clone/type");
function ApplyUncapitalize(value) {
    const [first, rest] = [value.slice(0, 1), value.slice(1)];
    return [first.toLowerCase(), rest].join('');
}
function ApplyCapitalize(value) {
    const [first, rest] = [value.slice(0, 1), value.slice(1)];
    return [first.toUpperCase(), rest].join('');
}
function ApplyUppercase(value) {
    return value.toUpperCase();
}
function ApplyLowercase(value) {
    return value.toLowerCase();
}
function FromTemplateLiteral(schema, mode) {
    const expression = (0, index_1.TemplateLiteralParseExact)(schema.pattern);
    const finite = (0, index_1.IsTemplateLiteralFinite)(expression);
    if (!finite)
        return { ...schema, pattern: FromLiteralValue(schema.pattern, mode) };
    const strings = [...(0, index_1.TemplateLiteralGenerate)(expression)];
    const literals = strings.map((value) => (0, index_2.Literal)(value));
    const mapped = FromRest(literals, mode);
    const union = (0, index_3.Union)(mapped);
    return (0, index_1.TemplateLiteral)([union]);
}
function FromLiteralValue(value, mode) {
    return (typeof value === 'string' ? (mode === 'Uncapitalize' ? ApplyUncapitalize(value) :
        mode === 'Capitalize' ? ApplyCapitalize(value) :
            mode === 'Uppercase' ? ApplyUppercase(value) :
                mode === 'Lowercase' ? ApplyLowercase(value) :
                    value) : value.toString());
}
function FromRest(T, mode) {
    const [L, ...R] = T;
    return (T.length > 0 ? [IntrinsicResolve(L, mode), ...FromRest(R, mode)] : []);
}
function IntrinsicResolve(schema, mode) {
    return ((0, type_1.TTemplateLiteral)(schema) ? FromTemplateLiteral(schema, mode) :
        (0, type_1.TUnion)(schema) ? (0, index_3.Union)(FromRest(schema.anyOf, mode)) :
            (0, type_1.TLiteral)(schema) ? (0, index_2.Literal)(FromLiteralValue(schema.const, mode)) :
                schema);
}
exports.IntrinsicResolve = IntrinsicResolve;
function Uncapitalize(T, options = {}) {
    return (0, type_2.CloneType)(IntrinsicResolve(T, 'Uncapitalize'), options);
}
exports.Uncapitalize = Uncapitalize;
function Uppercase(T, options = {}) {
    return (0, type_2.CloneType)(IntrinsicResolve(T, 'Uppercase'), options);
}
exports.Uppercase = Uppercase;
function Lowercase(T, options = {}) {
    return (0, type_2.CloneType)(IntrinsicResolve(T, 'Lowercase'), options);
}
exports.Lowercase = Lowercase;
function Capitalize(T, options = {}) {
    return (0, type_2.CloneType)(IntrinsicResolve(T, 'Capitalize'), options);
}
exports.Capitalize = Capitalize;
