"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyOfStringResolvePattern = exports.KeyOfStringResolve = void 0;
const index_1 = require("../helpers/index");
const index_2 = require("../operators/index");
const type_1 = require("../guard/type");
function FromRest(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [KeyOfStringResolve(L), ...FromRest(R)]
        : []);
}
function FromIntersect(T, C = FromRest(T)) {
    return (0, index_2.OperatorUnionMany)(C);
}
function FromUnion(T) {
    const C = FromRest(T);
    return (0, index_2.OperatorIntersectMany)(C);
}
function FromTuple(T, I = '0') {
    const [_, ...R] = T;
    return (T.length > 0
        ? [I, ...FromTuple(R, (0, index_1.Increment)(I))]
        : []);
}
function FromArray(_) {
    return (['[number]']);
}
function FromProperties(T) {
    return (globalThis.Object.getOwnPropertyNames(T));
}
function FromPatternProperties(patternProperties) {
    if (!includePatternProperties)
        return [];
    const patternPropertyKeys = globalThis.Object.getOwnPropertyNames(patternProperties);
    return patternPropertyKeys.map(key => {
        return (key[0] === '^' && key[key.length - 1] === '$')
            ? key.slice(1, key.length - 1)
            : key;
    });
}
function KeyOfStringResolve(T) {
    return ((0, type_1.TIntersect)(T) ? FromIntersect(T.allOf) :
        (0, type_1.TUnion)(T) ? FromUnion(T.anyOf) :
            (0, type_1.TTuple)(T) ? FromTuple(T.items ?? []) :
                (0, type_1.TArray)(T) ? FromArray(T.items) :
                    (0, type_1.TObject)(T) ? FromProperties(T.properties) :
                        (0, type_1.TRecord)(T) ? FromPatternProperties(T.patternProperties) :
                            []);
}
exports.KeyOfStringResolve = KeyOfStringResolve;
let includePatternProperties = false;
function KeyOfStringResolvePattern(schema) {
    includePatternProperties = true;
    const keys = KeyOfStringResolve(schema);
    includePatternProperties = false;
    const pattern = keys.map((key) => `(${key})`);
    return `^(${pattern.join('|')})$`;
}
exports.KeyOfStringResolvePattern = KeyOfStringResolvePattern;
