"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Partial = exports.PartialResolve = void 0;
const index_1 = require("../optional/index");
const index_2 = require("../object/index");
const index_3 = require("../intersect/index");
const index_4 = require("../union/index");
const index_5 = require("../discard/index");
const index_6 = require("../symbols/index");
const type_1 = require("../clone/type");
const type_2 = require("../guard/type");
function FromIntersect(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [PartialResolve(L), ...FromIntersect(R)]
        : []);
}
function FromUnion(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [PartialResolve(L), ...FromUnion(R)]
        : []);
}
function FromProperties(T) {
    return globalThis.Object.getOwnPropertyNames(T).reduce((Acc, K) => {
        return { ...Acc, [K]: (0, index_1.Optional)(T[K]) };
    }, {});
}
function PartialResolve(T) {
    return ((0, type_2.TIntersect)(T) ? (0, index_3.Intersect)(FromIntersect(T.allOf)) :
        (0, type_2.TUnion)(T) ? (0, index_4.Union)(FromUnion(T.anyOf)) :
            (0, type_2.TObject)(T) ? (0, index_2.Object)(FromProperties(T.properties)) :
                (0, index_2.Object)({}));
}
exports.PartialResolve = PartialResolve;
function Partial(T, options = {}) {
    const D = (0, index_5.Discard)(T, [index_6.TransformKind, '$id', 'required']);
    const R = (0, type_1.CloneType)(PartialResolve(T), options);
    return { ...D, ...R };
}
exports.Partial = Partial;
