"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Record = exports.RecordResolve = void 0;
const index_1 = require("../object/index");
const index_2 = require("../never/index");
const index_3 = require("../union/index");
const index_4 = require("../template-literal/index");
const index_5 = require("../patterns/index");
const index_6 = require("../indexed/index");
const index_7 = require("../symbols/index");
const type_1 = require("../clone/type");
const value_1 = require("../guard/value");
const type_2 = require("../guard/type");
function FromPattern(pattern, T) {
    return { [index_7.Kind]: 'Record', type: 'object', patternProperties: { [pattern]: (0, type_1.CloneType)(T) } };
}
function FromKeys(keys, T) {
    const properties = keys.reduce((acc, key) => ({ ...acc, [key]: (0, type_1.CloneType)(T) }), {});
    return (0, index_1.Object)(properties, { [index_7.Hint]: 'Record' });
}
function FromTemplateLiteralKey(K, T) {
    const expression = (0, index_4.TemplateLiteralParseExact)(K.pattern);
    return ((0, index_4.IsTemplateLiteralFinite)(expression)
        ? FromKeys((0, index_6.IndexedKeyResolve)(K), T)
        : FromPattern(K.pattern, T));
}
function FromUnionKey(K, T) {
    return FromKeys((0, index_6.IndexedKeyResolve)((0, index_3.Union)(K)), T);
}
function FromLiteralKey(K, T) {
    return FromKeys([K.toString()], T);
}
function FromStringKey(K, T) {
    const pattern = (0, value_1.IsUndefined)(K.pattern) ? index_5.PatternStringExact : K.pattern;
    return FromPattern(pattern, T);
}
function FromIntegerKey(_, T) {
    return FromPattern(index_5.PatternNumberExact, T);
}
function FromNumberKey(_, T) {
    return FromPattern(index_5.PatternNumberExact, T);
}
function RecordResolve(K, T) {
    return ((0, type_2.TUnion)(K) ? FromUnionKey(K.anyOf, T) :
        (0, type_2.TTemplateLiteral)(K) ? FromTemplateLiteralKey(K, T) :
            (0, type_2.TLiteral)(K) ? FromLiteralKey(K.const, T) :
                (0, type_2.TString)(K) ? FromStringKey(K, T) :
                    (0, type_2.TInteger)(K) ? FromIntegerKey(K, T) :
                        (0, type_2.TNumber)(K) ? FromNumberKey(K, T) :
                            (0, index_2.Never)());
}
exports.RecordResolve = RecordResolve;
function Record(K, T, options = {}) {
    return (0, type_1.CloneType)(RecordResolve(K, T), options);
}
exports.Record = Record;
