"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateLiteralGenerate = exports.TemplateLiteralGenerateError = void 0;
function* GenerateReduce(buffer) {
    if (buffer.length === 1)
        return yield* buffer[0];
    for (const left of buffer[0]) {
        for (const right of GenerateReduce(buffer.slice(1))) {
            yield `${left}${right}`;
        }
    }
}
function* GenerateAnd(expression) {
    return yield* GenerateReduce(expression.expr.map((expr) => [...TemplateLiteralGenerate(expr)]));
}
function* GenerateOr(expression) {
    for (const expr of expression.expr)
        yield* TemplateLiteralGenerate(expr);
}
function* GenerateConst(expression) {
    return yield expression.const;
}
class TemplateLiteralGenerateError extends Error {
}
exports.TemplateLiteralGenerateError = TemplateLiteralGenerateError;
function* TemplateLiteralGenerate(expression) {
    return expression.type === 'and'
        ? yield* GenerateAnd(expression)
        : expression.type === 'or'
            ? yield* GenerateOr(expression)
            : expression.type === 'const'
                ? yield* GenerateConst(expression)
                : (() => {
                    throw new TemplateLiteralGenerateError('Unknown expression');
                })();
}
exports.TemplateLiteralGenerate = TemplateLiteralGenerate;
