import { type TAny } from '../any/index';
import { type TArray, type ArrayOptions } from '../array/index';
import { type TBoolean } from '../boolean/index';
import { type TComposite } from '../composite/index';
import { type TConst } from '../const/index';
import { type TDeref } from '../deref/index';
import { type TEnum, type TEnumKey, type TEnumValue } from '../enum/index';
import { type TExclude } from '../exclude/index';
import { type TExtends } from '../extends/index';
import { type TExtract } from '../extract/index';
import { TIndex, type IndexedKeyResolve } from '../indexed/index';
import { type IntegerOptions, type TInteger } from '../integer/index';
import { type IntersectOptions, type IntersectResolve } from '../intersect/index';
import { type TCapitalize, type TUncapitalize, type TLowercase, type TUppercase } from '../intrinsic/index';
import { type TKeyOf } from '../keyof/index';
import { type TLiteral, type TLiteralValue } from '../literal/index';
import { type TNever } from '../never/index';
import { type TNot } from '../not/index';
import { type TNull } from '../null/index';
import { type TNumber, type NumberOptions } from '../number/index';
import { type TObject, type TProperties, type ObjectOptions } from '../object/index';
import { type TOmit } from '../omit/index';
import { type TOptional } from '../optional/index';
import { type TPartial } from '../partial/index';
import { type TPick } from '../pick/index';
import { type TReadonly } from '../readonly/index';
import { type TReadonlyOptional } from '../readonly-optional/index';
import { type RecordResolve } from '../record/index';
import { type TRecursive, type TThis } from '../recursive/index';
import { type TRef } from '../ref/index';
import { type TRequired } from '../required/index';
import { type TRest } from '../rest/index';
import { type TSchema, type SchemaOptions } from '../schema/index';
import { type TString, type StringOptions } from '../string/index';
import { type TTemplateLiteral, type TTemplateLiteralKind, type TemplateLiteralSyntax } from '../template-literal/index';
import { TransformDecodeBuilder } from '../transform/index';
import { type TTuple } from '../tuple/index';
import { type UnionResolve } from '../union/index';
import { type TUnknown } from '../unknown/index';
import { type TUnsafe, type UnsafeOptions } from '../unsafe/index';
export declare class JsonTypeBuilder {
    Strict<T extends TSchema>(schema: T): T;
    ReadonlyOptional<T extends TSchema>(schema: T): TReadonlyOptional<T>;
    Readonly<T extends TSchema>(schema: T): TReadonly<T>;
    Optional<T extends TSchema>(schema: T): TOptional<T>;
    Any(options?: SchemaOptions): TAny;
    Array<T extends TSchema>(schema: T, options?: ArrayOptions): TArray<T>;
    Boolean(options?: SchemaOptions): TBoolean;
    Capitalize<T extends TSchema>(schema: T, options?: SchemaOptions): TCapitalize<T>;
    Composite<T extends TObject[]>(objects: [...T], options?: ObjectOptions): TComposite<T>;
    Const<T>(value: T, options?: SchemaOptions): TConst<T>;
    Deref<T extends TSchema>(schema: T, references: TSchema[]): TDeref<T>;
    Enum<V extends TEnumValue, T extends Record<TEnumKey, V>>(item: T, options?: SchemaOptions): TEnum<T>;
    Extends<L extends TSchema, R extends TSchema, T extends TSchema, U extends TSchema>(left: L, right: R, trueType: T, falseType: U, options?: SchemaOptions): TExtends<L, R, T, U>;
    Exclude<L extends TSchema, R extends TSchema>(unionType: L, excludedMembers: R, options?: SchemaOptions): TExclude<L, R>;
    Extract<L extends TSchema, R extends TSchema>(type: L, union: R, options?: SchemaOptions): TExtract<L, R>;
    Index<T extends TSchema, K extends TSchema, I extends PropertyKey[] = IndexedKeyResolve<K>>(T: T, K: K, options?: SchemaOptions): TIndex<T, I>;
    Index<T extends TSchema, K extends PropertyKey[]>(T: T, K: readonly [...K], options?: SchemaOptions): TIndex<T, K>;
    Integer(options?: IntegerOptions): TInteger;
    Intersect<T extends TSchema[]>(T: [...T], options?: IntersectOptions): IntersectResolve<T>;
    KeyOf<T extends TSchema>(schema: T, options?: SchemaOptions): TKeyOf<T>;
    Literal<T extends TLiteralValue>(value: T, options?: SchemaOptions): TLiteral<T>;
    Lowercase<T extends TSchema>(schema: T, options?: SchemaOptions): TLowercase<T>;
    Never(options?: SchemaOptions): TNever;
    Not<T extends TSchema>(schema: T, options?: SchemaOptions): TNot<T>;
    Null(options?: SchemaOptions): TNull;
    Number(options?: NumberOptions): TNumber;
    Object<T extends TProperties>(properties: T, options?: ObjectOptions): TObject<T>;
    Omit<T extends TSchema, K extends TSchema, I extends PropertyKey[] = IndexedKeyResolve<K>>(T: T, K: K, options?: SchemaOptions): TOmit<T, I>;
    Omit<T extends TSchema, K extends PropertyKey[]>(T: T, K: readonly [...K], options?: SchemaOptions): TOmit<T, K>;
    Partial<T extends TSchema>(schema: T, options?: ObjectOptions): TPartial<T>;
    Pick<T extends TSchema, K extends TSchema, I extends PropertyKey[] = IndexedKeyResolve<K>>(T: T, K: K, options?: SchemaOptions): TPick<T, I>;
    Pick<T extends TSchema, K extends PropertyKey[]>(T: T, K: readonly [...K], options?: SchemaOptions): TPick<T, K>;
    Record<K extends TSchema, T extends TSchema>(key: K, schema: T, options?: ObjectOptions): RecordResolve<K, T>;
    Recursive<T extends TSchema>(callback: (thisType: TThis) => T, options?: SchemaOptions): TRecursive<T>;
    Ref<T extends TSchema>(schema: T, options?: SchemaOptions): TRef<T>;
    Ref<T extends TSchema>($ref: string, options?: SchemaOptions): TRef<T>;
    Required<T extends TSchema>(schema: T, options?: SchemaOptions): TRequired<T>;
    Rest<T extends TSchema>(schema: T): TRest<T>;
    String(options?: StringOptions): TString;
    TemplateLiteral<T extends string>(syntax: T, options?: SchemaOptions): TemplateLiteralSyntax<T>;
    TemplateLiteral<T extends TTemplateLiteralKind[]>(kinds: [...T], options?: SchemaOptions): TTemplateLiteral<T>;
    Transform<I extends TSchema>(schema: I): TransformDecodeBuilder<I>;
    Tuple<T extends TSchema[]>(items: [...T], options?: SchemaOptions): TTuple<T>;
    Uncapitalize<T extends TSchema>(schema: T, options?: SchemaOptions): TUncapitalize<T>;
    Union<T extends TSchema[]>(schemas: [...T], options?: SchemaOptions): UnionResolve<T>;
    Unknown(options?: SchemaOptions): TUnknown;
    Unsafe<T>(options?: UnsafeOptions): TUnsafe<T>;
    Uppercase<T extends TSchema>(schema: T, options?: SchemaOptions): TUppercase<T>;
}
