"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasTransform = void 0;
const type_1 = require("../../type/guard/type");
const index_1 = require("../guard/index");
const index_2 = require("../deref/index");
const index_3 = require("../../type/symbols/index");
function TArray(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.items, references);
}
function TAsyncIterator(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.items, references);
}
function TConstructor(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.returns, references) || schema.parameters.some((schema) => Visit(schema, references));
}
function TFunction(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.returns, references) || schema.parameters.some((schema) => Visit(schema, references));
}
function TIntersect(schema, references) {
    return (0, type_1.TTransform)(schema) || (0, type_1.TTransform)(schema.unevaluatedProperties) || schema.allOf.some((schema) => Visit(schema, references));
}
function TIterator(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.items, references);
}
function TNot(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.not, references);
}
function TObject(schema, references) {
    return ((0, type_1.TTransform)(schema) ||
        Object.values(schema.properties).some((schema) => Visit(schema, references)) ||
        ((0, type_1.TSchema)(schema.additionalProperties) && Visit(schema.additionalProperties, references)));
}
function TPromise(schema, references) {
    return (0, type_1.TTransform)(schema) || Visit(schema.item, references);
}
function TRecord(schema, references) {
    const pattern = Object.getOwnPropertyNames(schema.patternProperties)[0];
    const property = schema.patternProperties[pattern];
    return (0, type_1.TTransform)(schema) || Visit(property, references) || ((0, type_1.TSchema)(schema.additionalProperties) && (0, type_1.TTransform)(schema.additionalProperties));
}
function TRef(schema, references) {
    if ((0, type_1.TTransform)(schema))
        return true;
    return Visit((0, index_2.Deref)(schema, references), references);
}
function TThis(schema, references) {
    if ((0, type_1.TTransform)(schema))
        return true;
    return Visit((0, index_2.Deref)(schema, references), references);
}
function TTuple(schema, references) {
    return (0, type_1.TTransform)(schema) || (!(0, index_1.IsUndefined)(schema.items) && schema.items.some((schema) => Visit(schema, references)));
}
function TUnion(schema, references) {
    return (0, type_1.TTransform)(schema) || schema.anyOf.some((schema) => Visit(schema, references));
}
function Visit(schema, references) {
    const references_ = (0, index_1.IsString)(schema.$id) ? [...references, schema] : references;
    const schema_ = schema;
    if (schema.$id && visited.has(schema.$id))
        return false;
    if (schema.$id)
        visited.add(schema.$id);
    switch (schema[index_3.Kind]) {
        case 'Array':
            return TArray(schema_, references_);
        case 'AsyncIterator':
            return TAsyncIterator(schema_, references_);
        case 'Constructor':
            return TConstructor(schema_, references_);
        case 'Function':
            return TFunction(schema_, references_);
        case 'Intersect':
            return TIntersect(schema_, references_);
        case 'Iterator':
            return TIterator(schema_, references_);
        case 'Not':
            return TNot(schema_, references_);
        case 'Object':
            return TObject(schema_, references_);
        case 'Promise':
            return TPromise(schema_, references_);
        case 'Record':
            return TRecord(schema_, references_);
        case 'Ref':
            return TRef(schema_, references_);
        case 'This':
            return TThis(schema_, references_);
        case 'Tuple':
            return TTuple(schema_, references_);
        case 'Union':
            return TUnion(schema_, references_);
        default:
            return (0, type_1.TTransform)(schema);
    }
}
const visited = new Set();
function HasTransform(schema, references) {
    visited.clear();
    return Visit(schema, references);
}
exports.HasTransform = HasTransform;
